@@ -36,9 +36,11 @@ import org.jaudiotagger.audio.ogg.OggFileWriter;
 import org.jaudiotagger.audio.real.RealFileReader;
 import org.jaudiotagger.audio.wav.WavFileReader;
 import org.jaudiotagger.audio.wav.WavFileWriter;
+import org.jaudiotagger.logging.ErrorMessage;
 import org.jaudiotagger.tag.TagException;
 
 import java.io.File;
+import java.io.FileNotFoundException;
 import java.io.IOException;
 import java.util.HashMap;
 import java.util.Iterator;
@@ -80,7 +82,7 @@ import java.util.logging.Logger;
  * </p>
  *
  * @author Raphael Slinckx
- * @version $Id: AudioFileIO.java,v 1.19 2009/11/12 15:43:00 paultaylor Exp $
+ * @version $Id: AudioFileIO.java 929 2010-11-17 12:36:46Z paultaylor $
  * @see AudioFile
  * @see org.jaudiotagger.tag.Tag
  * @since v0.01
@@ -106,7 +108,7 @@ public class AudioFileIO
      *
      * @param f The file where the tag will be deleted
      * @throws CannotWriteException If the file could not be written/accessed, the extension
-     *                              wasn't recognized, or other IO error occured.
+     *                              wasn't recognized, or other IO error occurred.
      * @throws org.jaudiotagger.audio.exceptions.CannotReadException
      */
     public static void delete(AudioFile f) throws CannotReadException, CannotWriteException
@@ -115,7 +117,7 @@ public class AudioFileIO
     }
 
     /**
-     * This method returns the default isntance for static use.<br>
+     * This method returns the default instance for static use.<br>
      *
      * @return The default instance.
      */
@@ -134,9 +136,9 @@ public class AudioFileIO
      * </p>
      *
      * @param f The file to read.
-     * @return The AudioFile with the file tag and the file encoding infos.
+     * @return The AudioFile with the file tag and the file encoding info.
      * @throws CannotReadException If the file could not be read, the extension wasn't
-     *                             recognized, or an IO error occured during the read.
+     *                             recognized, or an IO error occurred during the read.
      * @throws org.jaudiotagger.tag.TagException
      * @throws org.jaudiotagger.audio.exceptions.ReadOnlyFileException
      * @throws java.io.IOException
@@ -150,12 +152,12 @@ public class AudioFileIO
 
     /**
      * <p/>
-     * Write the tag contained in the audiofile in the actual file on the disk.
+     * Write the tag contained in the audioFile in the actual file on the disk.
      * </p>
      *
      * @param f The AudioFile to be written
      * @throws CannotWriteException If the file could not be written/accessed, the extension
-     *                              wasn't recognized, or other IO error occured.
+     *                              wasn't recognized, or other IO error occurred.
      */
     public static void write(AudioFile f) throws CannotWriteException
     {
@@ -200,7 +202,7 @@ public class AudioFileIO
      *
      * @param f The file where the tag will be deleted
      * @throws CannotWriteException If the file could not be written/accessed, the extension
-     *                              wasn't recognized, or other IO error occured.
+     *                              wasn't recognized, or other IO error occurred.
      * @throws org.jaudiotagger.audio.exceptions.CannotReadException
      */
     public void deleteTag(AudioFile f) throws CannotReadException, CannotWriteException
@@ -210,8 +212,7 @@ public class AudioFileIO
         Object afw = writers.get(ext);
         if (afw == null)
         {
-            throw new CannotWriteException(
-                    "No Deleter associated to this extension: " + ext);
+            throw new CannotWriteException(ErrorMessage.NO_DELETER_FOR_THIS_FORMAT.getMsg(ext));
         }
 
         ((AudioFileWriter) afw).delete(f);
@@ -225,7 +226,7 @@ public class AudioFileIO
 
         // Tag Readers
         readers.put(SupportedFileFormat.OGG.getFilesuffix(), new OggFileReader());
-        readers.put(SupportedFileFormat.FLAC.getFilesuffix(), new FlacFileReader());
+        readers.put(SupportedFileFormat.FLAC.getFilesuffix(),new FlacFileReader());
         readers.put(SupportedFileFormat.MP3.getFilesuffix(), new MP3FileReader());
         readers.put(SupportedFileFormat.MP4.getFilesuffix(), new Mp4FileReader());
         readers.put(SupportedFileFormat.M4A.getFilesuffix(), new Mp4FileReader());
@@ -262,9 +263,9 @@ public class AudioFileIO
      * </p>
      *
      * @param f The file to read.
-     * @return The AudioFile with the file tag and the file encoding infos.
+     * @return The AudioFile with the file tag and the file encoding info.
      * @throws CannotReadException If the file could not be read, the extension wasn't
-     *                             recognized, or an IO error occured during the read.
+     *                             recognized, or an IO error occurred during the read.
      * @throws org.jaudiotagger.tag.TagException
      * @throws org.jaudiotagger.audio.exceptions.ReadOnlyFileException
      * @throws java.io.IOException
@@ -273,18 +274,33 @@ public class AudioFileIO
     public AudioFile readFile(File f)
             throws CannotReadException, IOException, TagException, ReadOnlyFileException, InvalidAudioFrameException
     {
+        checkFileExists(f);
         String ext = Utils.getExtension(f);
 
         AudioFileReader afr = readers.get(ext);
         if (afr == null)
         {
-            throw new CannotReadException(
-                    "No Reader associated to this extension: " + ext);
+            throw new CannotReadException(ErrorMessage.NO_READER_FOR_THIS_FORMAT.getMsg(ext));
         }
 
         return afr.read(f);
     }
 
+    /**
+     * Check does file exist
+     *
+     * @param file
+     * @throws java.io.FileNotFoundException
+     */
+    public void checkFileExists(File file)throws FileNotFoundException
+    {
+        logger.info("Reading file:" + "path" + file.getPath() + ":abs:" + file.getAbsolutePath());
+        if (!file.exists())
+        {
+            logger.severe("Unable to find:" + file.getPath());
+            throw new FileNotFoundException(ErrorMessage.UNABLE_TO_FIND_FILE.getMsg(file.getPath()));
+        }
+    }
     /**
      * Removes an listener for all file formats.
      *
@@ -298,12 +314,12 @@ public class AudioFileIO
 
     /**
      * <p/>
-     * Write the tag contained in the audiofile in the actual file on the disk.
+     * Write the tag contained in the audioFile in the actual file on the disk.
      * </p>
      *
      * @param f The AudioFile to be written
      * @throws CannotWriteException If the file could not be written/accessed, the extension
-     *                              wasn't recognized, or other IO error occured.
+     *                              wasn't recognized, or other IO error occurred.
      */
     public void writeFile(AudioFile f) throws CannotWriteException
     {
@@ -312,7 +328,7 @@ public class AudioFileIO
         AudioFileWriter afw = writers.get(ext);
         if (afw == null)
         {
-            throw new CannotWriteException("No Writer associated to this extension: " + ext);
+            throw new CannotWriteException(ErrorMessage.NO_WRITER_FOR_THIS_FORMAT.getMsg(ext));
         }
 
         afw.write(f);
