@@ -1,6 +1,6 @@
 /*
  * Entagged Audio Tag library
- * Copyright (c) 2003-2005 Raphal Slinckx <raphael@slinckx.net>
+ * Copyright (c) 2003-2005 Raphaël Slinckx <raphael@slinckx.net>
  * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -112,9 +112,13 @@ public class Mp4TagWriter
      * we have to adjust the size of the size field upto the moovheader level for the udta atom and
      * its child meta atom.
      *
+     * @param moovHeader
      * @param moovBuffer
      * @param sizeAdjustment can be negative or positive     *
+     * @param udtaHeader
+     * @param metaHeader
      * @return
+     * @throws java.io.IOException
      */
     private void adjustSizeOfMoovHeader
             (Mp4BoxHeader moovHeader,
@@ -186,14 +190,14 @@ public class Mp4TagWriter
         int oldIlstSize = 0;
         int relativeIlstposition;
         int relativeIlstEndPosition;
-        int startIlstWithinFile=0;
+        int startIlstWithinFile;
         int newIlstSize;
-        int oldMetaLevelFreeAtomSize = 0;
-        long extraDataSize = 0;
+        int oldMetaLevelFreeAtomSize;
+        long extraDataSize;
         int level1SearchPosition = 0;
-        int topLevelFreePosition = 0;
-        int topLevelFreeSize = 0;
-        boolean topLevelFreeAtomComesBeforeMdatAtom = true;
+        int topLevelFreePosition;
+        int topLevelFreeSize;
+        boolean topLevelFreeAtomComesBeforeMdatAtom;
         Mp4BoxHeader topLevelFreeHeader;
 
         Mp4AtomTree atomTree;
@@ -219,6 +223,13 @@ public class Mp4TagWriter
         Mp4BoxHeader hdlrMetaHeader     = atomTree.getBoxHeader(atomTree.getHdlrWithinMetaNode());
         Mp4BoxHeader hdlrMdiaHeader     = atomTree.getBoxHeader(atomTree.getHdlrWithinMdiaNode());
         Mp4BoxHeader mdatHeader         = atomTree.getBoxHeader(atomTree.getMdatNode());
+
+        //Unable to find audio so no chance of saving any changes
+        if(mdatHeader==null)
+        {
+            throw new CannotWriteException(ErrorMessage.MP4_CHANGES_TO_FILE_FAILED_CANNOT_FIND_AUDIO.getMsg());    
+        }
+
         Mp4BoxHeader trakHeader         = (Mp4BoxHeader)atomTree.getTrakNodes().get(0).getUserObject();
 
         ByteBuffer   moovBuffer         = atomTree.getMoovBuffer();
@@ -262,8 +273,8 @@ public class Mp4TagWriter
             else
             {
                 //There no ilst or meta header so we set to position where it would be if it existed
-                relativeIlstposition = (int) (moovHeader.getLength() - Mp4BoxHeader.HEADER_LENGTH);
-                relativeIlstEndPosition = relativeIlstposition + ilstHeader.getLength();
+                relativeIlstposition = moovHeader.getLength() - Mp4BoxHeader.HEADER_LENGTH;
+                relativeIlstEndPosition = relativeIlstposition ;
                 startIlstWithinFile = (int)(moovHeader.getFilePos() + moovHeader.getLength());
             }
         }
@@ -281,7 +292,7 @@ public class Mp4TagWriter
             else
             {
                 //There no udta,ilst or meta header so we set to position where it would be if it existed
-                relativeIlstposition = (int) (moovHeader.getLength() - Mp4BoxHeader.HEADER_LENGTH);
+                relativeIlstposition = moovHeader.getLength() - Mp4BoxHeader.HEADER_LENGTH;
                 relativeIlstEndPosition = relativeIlstposition;
                 startIlstWithinFile = (int)(moovHeader.getFilePos() + moovHeader.getLength());
             }
