@@ -10,56 +10,79 @@ package org.jaudiotagger.tag;
  */
 public enum FieldKey
 {
-    ARTIST,
     ALBUM,
-    TITLE,
-    TRACK,
-    YEAR,
-    GENRE,
-    COMMENT,
     ALBUM_ARTIST,
-    COMPOSER,
-    GROUPING,
-    DISC_NO,
-    COVER_ART,
-    BPM,
-    MUSICBRAINZ_ARTISTID,
-    MUSICBRAINZ_RELEASEID,
-    MUSICBRAINZ_RELEASEARTISTID,
-    MUSICBRAINZ_TRACK_ID,
-    MUSICBRAINZ_DISC_ID,
-    MUSICIP_ID,
-    AMAZON_ID,
-    MUSICBRAINZ_RELEASE_STATUS,
-    MUSICBRAINZ_RELEASE_TYPE,
-    MUSICBRAINZ_RELEASE_COUNTRY,
-    LYRICS,
-    IS_COMPILATION,
-    ARTIST_SORT,
     ALBUM_ARTIST_SORT,
     ALBUM_SORT,
-    TITLE_SORT,
-    COMPOSER_SORT,
-    ENCODER,
-    ISRC,
+    AMAZON_ID,
+    ARTIST,
+    ARTIST_SORT,
     BARCODE,
+    BPM,
     CATALOG_NO,
-    RECORD_LABEL,
-    LYRICIST,
+    COMMENT,
+    COMPOSER,
+    COMPOSER_SORT,
     CONDUCTOR,
-    REMIXER,
-    MOOD,
-    MEDIA,
-    URL_OFFICIAL_RELEASE_SITE,
-    URL_DISCOGS_RELEASE_SITE,
-    URL_WIKIPEDIA_RELEASE_SITE,
-    URL_OFFICIAL_ARTIST_SITE,
-    URL_DISCOGS_ARTIST_SITE,
-    URL_WIKIPEDIA_ARTIST_SITE,
-    LANGUAGE,
-    KEY,
-    URL_LYRICS_SITE,
-    TRACK_TOTAL,
+    COVER_ART,
+    CUSTOM1,
+    CUSTOM2,
+    CUSTOM3,
+    CUSTOM4,
+    CUSTOM5,
+    DISC_NO,
     DISC_TOTAL,
+    ENCODER,
+    FBPM,
+    GENRE,
+    GROUPING,
+    ISRC,
+    IS_COMPILATION,
+    KEY,
+    LANGUAGE,
+    LYRICIST,
+    LYRICS,
+    MEDIA,
+    MOOD,
+    MUSICBRAINZ_ARTISTID,
+    MUSICBRAINZ_DISC_ID,
+    MUSICBRAINZ_RELEASEARTISTID,
+    MUSICBRAINZ_RELEASEID,
+    MUSICBRAINZ_RELEASE_COUNTRY,
+    MUSICBRAINZ_RELEASE_GROUP_ID,
+    MUSICBRAINZ_RELEASE_STATUS,
+    MUSICBRAINZ_RELEASE_TYPE,
+    MUSICBRAINZ_TRACK_ID,
+    MUSICBRAINZ_WORK_ID,
+    MUSICIP_ID,
+    OCCASION,
+    ORIGINAL_ALBUM,
+    ORIGINAL_ARTIST,
+    ORIGINAL_LYRICIST,
+    ORIGINAL_YEAR,
+    QUALITY,
+    RATING,
+    RECORD_LABEL,
+    REMIXER,
+    SCRIPT,
+    TAGS,
+    TEMPO,
+    TITLE,
+    TITLE_SORT,
+    TRACK,
+    TRACK_TOTAL,
+    URL_DISCOGS_ARTIST_SITE,
+    URL_DISCOGS_RELEASE_SITE,
+    URL_LYRICS_SITE,
+    URL_OFFICIAL_ARTIST_SITE,
+    URL_OFFICIAL_RELEASE_SITE,
+    URL_WIKIPEDIA_ARTIST_SITE,
+    URL_WIKIPEDIA_RELEASE_SITE,
+    YEAR,
+    ENGINEER,
+    PRODUCER,
+    DJMIXER,
+    MIXER,
+    ARRANGER,
     ;
 }
