@@ -2,7 +2,7 @@
  *  @author : Paul Taylor
  *  @author : Eric Farng
  *
- *  Version @version:$Id: TagOptionSingleton.java,v 1.19 2009/11/12 15:43:03 paultaylor Exp $
+ *  Version @version:$Id: TagOptionSingleton.java 929 2010-11-17 12:36:46Z paultaylor $
  *
  *  MusicTag Copyright (C)2003,2004
  *
@@ -237,10 +237,16 @@ public class TagOptionSingleton
     private boolean truncateTextWithoutErrors = false;
 
     /**
-     * Frames such as TRCK and TPOS sometimes pad single didgit numbers to aid sorting
+     * Frames such as TRCK and TPOS sometimes pad single digit numbers to aid sorting
      */
     private boolean padNumbers = false;
 
+    /**
+     * There are a couple of problems with the Java implemenatation ono Google Android, enabling this value
+     * switches on Google workarounds
+     */
+    private boolean isAndroid = false;
+
     /**
      * Creates a new TagOptions datatype. All Options are set to their default
      * values
@@ -758,6 +764,7 @@ public class TagOptionSingleton
         resetTextEncodingForExistingFrames = false;
         truncateTextWithoutErrors = false;
         padNumbers = false;
+        isAndroid = false;
 
         //default all lyrics3 fields to save. id3v1 fields are individual
         // settings. id3v2 fields are always looked at to save.
@@ -914,9 +921,9 @@ public class TagOptionSingleton
     }
 
     /**
-     * Do we remove unneccessary trailing null characters on write
+     * Do we remove unnecessary trailing null characters on write
      *
-     * @return true if we remove unneccessary trailing null characters on write
+     * @return true if we remove unnecessary trailing null characters on write
      */
     public boolean isRemoveTrailingTerminatorOnWrite()
     {
@@ -924,7 +931,7 @@ public class TagOptionSingleton
     }
 
     /**
-     * Remove unneccessary trailing null characters on write
+     * Remove unnecessary trailing null characters on write
      *
      * @param removeTrailingTerminatorOnWrite
      *
@@ -1063,4 +1070,14 @@ public class TagOptionSingleton
     {
         this.padNumbers = padNumbers;
     }
+
+    public boolean isAndroid()
+    {
+        return isAndroid;
+    }
+
+    public void setAndroid(boolean android)
+    {
+        isAndroid = android;
+    }
 }
