@@ -2,7 +2,7 @@
  *  @author : Paul Taylor
  *  @author : Eric Farng
  *
- *  Version @version:$Id: AbstractDataType.java,v 1.19 2009/11/12 15:42:55 paultaylor Exp $
+ *  Version @version:$Id: AbstractDataType.java 929 2010-11-17 12:36:46Z paultaylor $
  *
  *  MusicTag Copyright (C)2003,2004
  *
@@ -72,7 +72,7 @@ public abstract class AbstractDataType
     protected AbstractDataType(String identifier, AbstractTagFrameBody frameBody)
     {
         this.identifier = identifier;
-        this.frameBody = frameBody;
+        this.frameBody  = frameBody;
     }
 
     /**
@@ -85,7 +85,7 @@ public abstract class AbstractDataType
     protected AbstractDataType(String identifier, AbstractTagFrameBody frameBody, Object value)
     {
         this.identifier = identifier;
-        this.frameBody = frameBody;
+        this.frameBody  = frameBody;
         setValue(value);
     }
 
@@ -143,6 +143,10 @@ public abstract class AbstractDataType
         {
             this.value = copyObject.value;
         }
+        else if(copyObject.value instanceof PairedTextEncodedStringNullTerminated.ValuePairs)
+        {
+            this.value = copyObject.value;
+        }
         else if(copyObject.value instanceof PartOfSet.PartOfSetValue)
         {
             this.value = copyObject.value;
@@ -267,6 +271,11 @@ public abstract class AbstractDataType
      */
     public boolean equals(Object obj)
     {
+        if(this==obj)
+        {
+            return true;
+        }
+
         if (!(obj instanceof AbstractDataType))
         {
             return false;
