@@ -8,6 +8,7 @@ import org.jaudiotagger.tag.id3.valuepair.ImageFormats;
 import org.jaudiotagger.tag.id3.valuepair.TextEncoding;
 import org.jaudiotagger.tag.reference.PictureTypes;
 import org.jaudiotagger.tag.vorbiscomment.VorbisCommentTag;
+import org.jaudiotagger.tag.vorbiscomment.VorbisCommentFieldKey;
 import org.jaudiotagger.logging.ErrorMessage;
 
 import javax.imageio.ImageIO;
@@ -64,7 +65,7 @@ public class FlacTag implements Tag
         }
     }
 
-    public List<TagField> get(String id)
+    public List<TagField> getFields(String id)
     {
         if (id.equals(FieldKey.COVER_ART.name()))
         {
@@ -77,7 +78,7 @@ public class FlacTag implements Tag
         }
         else
         {
-            return tag.get(id);
+            return tag.getFields(id);
         }
     }
 
@@ -123,7 +124,34 @@ public class FlacTag implements Tag
         TagField tagfield = createField(genericKey,value);
         addField(tagfield);
     }
-    
+
+    /**
+     * Create and set field with name of vorbisCommentkey
+     *
+     * @param vorbisCommentKey
+     * @param value
+     * @throws KeyNotFoundException
+     * @throws FieldDataInvalidException
+     */
+    public void setField(String vorbisCommentKey, String value) throws KeyNotFoundException, FieldDataInvalidException
+    {
+        TagField tagfield = createField(vorbisCommentKey,value);
+        setField(tagfield);
+    }
+
+    /**
+     * Create and add field with name of vorbisCommentkey
+     * @param vorbisCommentKey
+     * @param value
+     * @throws KeyNotFoundException
+     * @throws FieldDataInvalidException
+     */
+    public void addField(String vorbisCommentKey, String value) throws KeyNotFoundException, FieldDataInvalidException
+    {
+        TagField tagfield = createField(vorbisCommentKey,value);
+        addField(tagfield);
+    }
+
     /**
      * @param field
      * @throws FieldDataInvalidException
@@ -159,6 +187,43 @@ public class FlacTag implements Tag
         }
     }
 
+    /**
+     * Create Tag Field using ogg key
+     *
+     * @param vorbisCommentFieldKey
+     * @param value
+     * @return
+     * @throws org.jaudiotagger.tag.KeyNotFoundException
+     * @throws org.jaudiotagger.tag.FieldDataInvalidException
+     */
+    public TagField createField(VorbisCommentFieldKey vorbisCommentFieldKey, String value) throws KeyNotFoundException,FieldDataInvalidException
+    {
+        if (vorbisCommentFieldKey.equals(VorbisCommentFieldKey.COVERART))
+        {
+            throw new UnsupportedOperationException(ErrorMessage.ARTWORK_CANNOT_BE_CREATED_WITH_THIS_METHOD.getMsg());
+        }
+        return tag.createField(vorbisCommentFieldKey,value);
+    }
+
+    /**
+     * Create Tag Field using ogg key
+     * <p/>
+     * This method is provided to allow you to create key of any value because VorbisComment allows
+     * arbitary keys.
+     *
+     * @param vorbisCommentFieldKey
+     * @param value
+     * @return
+     */
+    public TagField createField(String vorbisCommentFieldKey, String value)
+    {
+        if (vorbisCommentFieldKey.equals(VorbisCommentFieldKey.COVERART.getFieldName()))
+        {
+            throw new UnsupportedOperationException(ErrorMessage.ARTWORK_CANNOT_BE_CREATED_WITH_THIS_METHOD.getMsg());
+        }
+        return tag.createField(vorbisCommentFieldKey,value);
+    }
+
     public String getFirst(String id)
     {
         if (id.equals(FieldKey.COVER_ART.name()))
@@ -171,7 +236,20 @@ public class FlacTag implements Tag
         }
     }
 
-    public String getFirst(FieldKey id) throws KeyNotFoundException
+     /**
+     * The m parameter is effectively ignored
+     *
+     * @param id
+     * @param n
+     * @param m
+     * @return
+     */
+    public String getSubValue(FieldKey id, int n, int m)
+    {
+        return getValue(id,n);
+    }
+
+    public String getValue(FieldKey id,int index) throws KeyNotFoundException
     {
         if (id.equals(FieldKey.COVER_ART))
         {
@@ -179,9 +257,13 @@ public class FlacTag implements Tag
         }
         else
         {
-            return tag.getFirst(id);
+            return tag.getValue(id, index);
         }
+    }
 
+    public String getFirst(FieldKey id) throws KeyNotFoundException
+    {
+        return getValue(id,0);
     }
 
     public TagField getFirstField(String id)
@@ -237,6 +319,19 @@ public class FlacTag implements Tag
         }
     }
 
+    public void deleteField(String id) throws KeyNotFoundException
+      {
+          if (id.equals(FieldKey.COVER_ART.name()))
+          {
+              images.clear();
+          }
+          else
+          {
+              tag.deleteField(id);
+          }
+      }
+
+
     //TODO addField images to iterator
     public Iterator<TagField> getFields()
     {
@@ -248,6 +343,11 @@ public class FlacTag implements Tag
         return tag.getFieldCount() + images.size();
     }
 
+    public int getFieldCountIncludingSubValues()
+    {
+       return getFieldCount();
+    }
+
     public boolean setEncoding(String enc) throws FieldDataInvalidException
     {
         return tag.setEncoding(enc);
@@ -374,19 +474,7 @@ public class FlacTag implements Tag
 
         for(MetadataBlockDataPicture coverArt:images)
         {
-            Artwork artwork = new Artwork();
-            artwork.setMimeType(coverArt.getMimeType());
-            artwork.setDescription(coverArt.getDescription());
-            artwork.setPictureType(coverArt.getPictureType());
-            if(coverArt.isImageUrl())
-            {
-                artwork.setLinked(coverArt.isImageUrl());
-                artwork.setImageUrl(coverArt.getImageUrl());
-            }
-            else
-            {
-                artwork.setBinaryData(coverArt.getImageData());
-            }
+            Artwork artwork=Artwork.createArtworkFromMetadataBlockDataPicture(coverArt);
             artworkList.add(artwork);
         }
         return artworkList;
