@@ -16,12 +16,12 @@
 package org.jaudiotagger.tag.id3;
 
 import org.jaudiotagger.audio.mp3.MP3File;
-import org.jaudiotagger.tag.InvalidFrameException;
-import org.jaudiotagger.tag.InvalidTagException;
-import org.jaudiotagger.tag.TagField;
-import org.jaudiotagger.tag.TagOptionSingleton;
+import org.jaudiotagger.tag.*;
 import org.jaudiotagger.tag.id3.framebody.AbstractID3v2FrameBody;
+import org.jaudiotagger.tag.id3.framebody.FrameBodyEncrypted;
 import org.jaudiotagger.tag.id3.framebody.FrameBodyUnsupported;
+import org.jaudiotagger.tag.id3.valuepair.TextEncoding;
+import org.jaudiotagger.utils.EqualsUtil;
 
 import java.io.ByteArrayOutputStream;
 import java.lang.reflect.Constructor;
@@ -34,9 +34,9 @@ import java.util.logging.Level;
  *
  * @author : Paul Taylor
  * @author : Eric Farng
- * @version $Id: AbstractID3v2Frame.java,v 1.30 2009/11/12 15:42:57 paultaylor Exp $
+ * @version $Id: AbstractID3v2Frame.java 929 2010-11-17 12:36:46Z paultaylor $
  */
-public abstract class AbstractID3v2Frame extends AbstractTagFrame implements TagField
+public abstract class AbstractID3v2Frame extends AbstractTagFrame implements TagTextField
 {
 
     protected static final String TYPE_FRAME = "frame";
@@ -54,6 +54,24 @@ public abstract class AbstractID3v2Frame extends AbstractTagFrame implements Tag
     //when processing many files
     private String loggingFilename = "";
 
+    /**
+     *
+     * @return size in bytes of the frameid field
+     */
+    protected abstract int getFrameIdSize();
+
+    /**
+     *
+     * @return the size in bytes of the frame size field
+     */
+    protected abstract int getFrameSizeSize();
+
+    /**
+     *
+     * @return the size in bytes of the frame header
+     */
+    protected abstract int getFrameHeaderSize();
+
     /**
      * Create an empty frame
      */
@@ -186,17 +204,59 @@ public abstract class AbstractID3v2Frame extends AbstractTagFrame implements Tag
 
     }
 
+    /**
+     * Read the frameBody when frame marked as encrypted
+     *
+     * @param identifier
+     * @param byteBuffer
+     * @param frameSize
+     * @return
+     * @throws InvalidFrameException
+     * @throws InvalidDataTypeException
+     * @throws InvalidTagException
+     */
+    protected AbstractID3v2FrameBody readEncryptedBody(String identifier, ByteBuffer byteBuffer, int frameSize)
+            throws InvalidFrameException, InvalidDataTypeException 
+    {
+        try
+        {
+            AbstractID3v2FrameBody frameBody = new  FrameBodyEncrypted(identifier,byteBuffer, frameSize);
+            frameBody.setHeader(this);
+            return frameBody;
+        }
+        catch(InvalidTagException ite)
+        {
+            throw new InvalidDataTypeException(ite);
+        }
+    }
+
+    protected boolean isPadding(byte[] buffer)
+    {
+        if(
+                (buffer[0]=='\0')&&
+                (buffer[1]=='\0')&&
+                (buffer[2]=='\0')&&
+                (buffer[3]=='\0')
+           )
+        {
+            return true;
+        }
+        return false;
+    }
+
     /**
      * Read the frame body from the specified file via the buffer
      *
      * @param identifier the frame identifier
-     * @param byteBuffer to read the frabe body from
+     * @param byteBuffer to read the frame body from
      * @param frameSize
      * @return a newly created FrameBody
      * @throws InvalidFrameException unable to construct a framebody from the data
      */
     @SuppressWarnings("unchecked")
-    protected AbstractID3v2FrameBody readBody(String identifier, ByteBuffer byteBuffer, int frameSize) throws InvalidFrameException
+    //TODO using reflection is rather slow perhaps we should change this
+    protected AbstractID3v2FrameBody readBody(String identifier, ByteBuffer byteBuffer, int frameSize)
+            throws InvalidFrameException, InvalidDataTypeException
     {
         //Use reflection to map id to frame body, which makes things much easier
         //to keep things up to date,although slight performance hit.
@@ -243,6 +303,14 @@ public abstract class AbstractID3v2Frame extends AbstractTagFrame implements Tag
             {
                 throw (RuntimeException) ite.getCause();
             }
+            else if(ite.getCause() instanceof  InvalidFrameException )
+            {
+                throw (InvalidFrameException)ite.getCause();
+            }
+            else if(ite.getCause() instanceof  InvalidDataTypeException )
+            {
+                throw (InvalidDataTypeException)ite.getCause();
+            }
             else
             {
                 throw new InvalidFrameException(ite.getCause().getMessage());
@@ -271,6 +339,37 @@ public abstract class AbstractID3v2Frame extends AbstractTagFrame implements Tag
         return frameBody;
     }
 
+    /**
+     * Get the next frame id, throwing an exception if unable to do this and check against just having padded data
+     * 
+     * @param byteBuffer
+     * @return
+     * @throws PaddingException
+     * @throws InvalidFrameException
+     */
+    protected String readIdentifier(ByteBuffer byteBuffer) throws PaddingException,InvalidFrameException
+    {
+        byte[] buffer = new byte[getFrameIdSize()];
+
+        if (byteBuffer.position() + getFrameHeaderSize() >= byteBuffer.limit())
+        {
+            logger.warning(getLoggingFilename() + ":" + "No space to find another frame:");
+            throw new InvalidFrameException(getLoggingFilename() + ":" + "No space to find another frame");
+        }
+
+        //Read the Frame Identifier
+        byteBuffer.get(buffer, 0, getFrameIdSize());
+
+        if(isPadding(buffer))
+        {
+            throw new PaddingException(getLoggingFilename() + ":only padding found");
+        }
+
+        identifier = new String(buffer);
+        logger.fine(getLoggingFilename() + ":" + "Identifier is" + identifier);
+        return identifier;
+    }
+
     /**
      * This creates a new body based of type identifier but populated by the data
      * in the body. This is a different type to the body being created which is why
@@ -289,6 +388,7 @@ public abstract class AbstractID3v2Frame extends AbstractTagFrame implements Tag
      * @throws InvalidFrameException if unable to construct a framebody for the identifier and body provided.
      */
     @SuppressWarnings("unchecked")
+    //TODO using reflection is rather slow perhaps we should change this
     protected AbstractID3v2FrameBody readBody(String identifier, AbstractID3v2FrameBody body) throws InvalidFrameException
     {
         /* Use reflection to map id to frame body, which makes things much easier
@@ -379,17 +479,17 @@ public abstract class AbstractID3v2Frame extends AbstractTagFrame implements Tag
         return false;
     }
 
-    protected StatusFlags getStatusFlags()
+    public StatusFlags getStatusFlags()
     {
         return statusFlags;
     }
 
-    protected EncodingFlags getEncodingFlags()
+    public EncodingFlags getEncodingFlags()
     {
         return encodingFlags;
     }
 
-    class StatusFlags
+    public class StatusFlags
     {
         protected static final String TYPE_FLAGS = "statusFlags";
 
@@ -423,7 +523,22 @@ public abstract class AbstractID3v2Frame extends AbstractTagFrame implements Tag
         {
         }
 
+        public boolean equals(Object obj)
+        {
+            if ( this == obj ) return true;
 
+            if (!(obj instanceof StatusFlags))
+            {
+                return false;
+            }
+            StatusFlags that = (StatusFlags) obj;
+
+
+            return
+                  EqualsUtil.areEqual(this.getOriginalFlags(), that.getOriginalFlags()) &&
+                  EqualsUtil.areEqual(this.getWriteFlags(), that.getWriteFlags()) ;
+
+        }
     }
 
     class EncodingFlags
@@ -460,6 +575,21 @@ public abstract class AbstractID3v2Frame extends AbstractTagFrame implements Tag
         public void createStructure()
         {
         }
+
+        public boolean equals(Object obj)
+        {
+            if ( this == obj ) return true;
+
+            if (!(obj instanceof EncodingFlags))
+            {
+                return false;
+            }
+            EncodingFlags that = (EncodingFlags) obj;
+
+
+            return EqualsUtil.areEqual(this.getFlags(), that.getFlags());
+
+        }
     }
 
     /**
@@ -470,4 +600,50 @@ public abstract class AbstractID3v2Frame extends AbstractTagFrame implements Tag
         MP3File.getStructureFormatter().openHeadingElement(TYPE_FRAME, getIdentifier());
         MP3File.getStructureFormatter().closeHeadingElement(TYPE_FRAME);
     }
+
+    public boolean equals(Object obj)
+    {
+        if ( this == obj ) return true;
+        if (!(obj instanceof AbstractID3v2Frame))
+        {
+            return false;
+        }
+
+        AbstractID3v2Frame that = (AbstractID3v2Frame) obj;
+        return super.equals(that);
+    }
+
+    /**
+     * Returns the content of the field.
+     *
+     * For frames consisting of different fields, this will return the value deemed to be most
+     * likely to be required
+     *
+     * @return Content
+     */
+    public String getContent()
+    {
+        return getBody().getUserFriendlyValue();
+    }
+
+    /**
+     * Returns the current used charset encoding.
+     *
+     * @return Charset encoding.
+     */
+    public String getEncoding()
+    {
+        return TextEncoding.getInstanceOf().getValueForId(this.getBody().getTextEncoding());
+    }
+
+    /**
+     * Sets the content of the field.
+     *
+     * @param content fields content.
+     */
+    public void setContent(String content)
+    {
+        throw new UnsupportedOperationException("Not implemeneted please use the generic tag methods for setting content");
+    }
+
 }
