@@ -28,7 +28,7 @@ import java.util.TreeSet;
  * Here we specify how frames are mapped between different Tag Versions
  *
  * @author Paul Taylor
- * @version $Id: ID3Frames.java,v 1.18 2009/11/12 15:42:57 paultaylor Exp $
+ * @version $Id: ID3Frames.java 929 2010-11-17 12:36:46Z paultaylor $
  */
 public abstract class ID3Frames extends AbstractStringStringValuePair
 {
@@ -73,7 +73,7 @@ public abstract class ID3Frames extends AbstractStringStringValuePair
     }
 
     /**
-     * Are multiple ocurrences of frame allowed
+     * Are multiple occurrences of frame allowed
      * @param frameID
      * @return
      */
@@ -91,6 +91,10 @@ public abstract class ID3Frames extends AbstractStringStringValuePair
         return supportedFrames.contains(frameID);
     }
 
+    public TreeSet<String> getSupportedFrames()
+    {
+        return supportedFrames;
+    }
     /**
      * @param frameID
      * @return true if frames with this id are considered common
