@@ -2,7 +2,7 @@
  *  @author : Paul Taylor
  *  @author : Eric Farng
  *
- *  Version @version:$Id: ID3v11Tag.java,v 1.32 2009/11/12 15:42:57 paultaylor Exp $
+ *  Version @version:$Id: ID3v11Tag.java 929 2010-11-17 12:36:46Z paultaylor $
  *
  *  MusicTag Copyright (C)2003,2004
  *
@@ -362,6 +362,36 @@ public class ID3v11Tag extends ID3v1Tag
         }
     }
 
+    public String getFirst(FieldKey genericKey)
+    {
+        switch (genericKey)
+        {
+            case ARTIST:
+                return getFirstArtist();
+
+            case ALBUM:
+                return getFirstAlbum();
+
+            case TITLE:
+                return getFirstTitle();
+
+            case GENRE:
+                return getFirstGenre();
+
+            case YEAR:
+                return getFirstYear();
+
+            case TRACK:
+                return getFirstTrack();
+
+            case COMMENT:
+                return getFirstComment();
+
+            default:
+                return "";
+        }
+    }
+
     public TagField getFirstField(String id)
     {
         List<TagField> results;
@@ -425,7 +455,7 @@ public class ID3v11Tag extends ID3v1Tag
 
 
     /**
-     * Find identifer within byteBuffer to indicate that a v11 tag exists within the buffer
+     * Find identifier within byteBuffer to indicate that a v11 tag exists within the buffer
      *
      * @param byteBuffer
      * @return true if find header for v11 tag within buffer
