@@ -2,7 +2,7 @@
  *  @author : Paul Taylor
  *  @author : Eric Farng
  *
- *  Version @version:$Id: ID3v1Tag.java,v 1.35 2009/11/12 15:42:57 paultaylor Exp $
+ *  Version @version:$Id: ID3v1Tag.java 929 2010-11-17 12:36:46Z paultaylor $
  *
  *  MusicTag Copyright (C)2003,2004
  *
@@ -214,7 +214,7 @@ public class ID3v1Tag extends AbstractID3v1Tag implements Tag
         //TODO
     }
 
-    public List<TagField> get(String id)
+    public List<TagField> getFields(String id)
     {
 
         if (FieldKey.ARTIST.name().equals(id))
@@ -249,6 +249,11 @@ public class ID3v1Tag extends AbstractID3v1Tag implements Tag
         return 6;
     }
 
+    public int getFieldCountIncludingSubValues()
+    {
+       return getFieldCount();
+    }
+
     protected List<TagField> returnFieldToList(ID3v1TagField field)
     {
         List<TagField> fields = new ArrayList<TagField>();
@@ -275,7 +280,7 @@ public class ID3v1Tag extends AbstractID3v1Tag implements Tag
      *
      * @return album
      */
-    private String getFirstAlbum()
+    protected String getFirstAlbum()
     {
         return album;
     }
@@ -318,7 +323,7 @@ public class ID3v1Tag extends AbstractID3v1Tag implements Tag
      *
      * @return artist
      */
-    private String getFirstArtist()
+    protected String getFirstArtist()
     {
         return artist;
     }
@@ -751,9 +756,6 @@ public class ID3v1Tag extends AbstractID3v1Tag implements Tag
             case YEAR:
                 return getFirstYear();
 
-            case TRACK:
-                return getFirstTrack();
-
             case COMMENT:
                 return getFirstComment();
 
@@ -762,6 +764,24 @@ public class ID3v1Tag extends AbstractID3v1Tag implements Tag
         }
     }
 
+     /**
+     * The m parameter is effectively ignored
+     *
+     * @param id
+     * @param n
+     * @param m
+     * @return
+     */
+    public String getSubValue(FieldKey id, int n, int m)
+    {
+        return getValue(id,n);
+    }
+
+    public String getValue(FieldKey genericKey, int index)
+    {
+        return getFirst(genericKey);
+    }
+
     /**
      * Delete any instance of tag fields with this key
      *
@@ -797,6 +817,15 @@ public class ID3v1Tag extends AbstractID3v1Tag implements Tag
         }
     }
 
+    public void deleteField(String id)
+    {
+        FieldKey key = FieldKey.valueOf(id);
+        if(key!=null)
+        {
+            deleteField(key);
+        }
+    }
+
     /**
      * @param obj
      * @return true if this and obj are equivalent
@@ -977,7 +1006,7 @@ public class ID3v1Tag extends AbstractID3v1Tag implements Tag
     }
 
     /**
-     * Create strcutured representation of this item.
+     * Create structured representation of this item.
      */
     public void createStructure()
     {
