@@ -17,12 +17,12 @@ package org.jaudiotagger.tag.id3;
 
 import org.jaudiotagger.audio.generic.Utils;
 import org.jaudiotagger.audio.mp3.MP3File;
-import org.jaudiotagger.tag.EmptyFrameException;
-import org.jaudiotagger.tag.InvalidFrameException;
-import org.jaudiotagger.tag.InvalidFrameIdentifierException;
+import org.jaudiotagger.tag.*;
 import org.jaudiotagger.tag.id3.framebody.AbstractID3v2FrameBody;
 import org.jaudiotagger.tag.id3.framebody.FrameBodyDeprecated;
 import org.jaudiotagger.tag.id3.framebody.FrameBodyUnsupported;
+import org.jaudiotagger.tag.id3.valuepair.TextEncoding;
+import org.jaudiotagger.utils.EqualsUtil;
 
 import java.io.ByteArrayOutputStream;
 import java.io.IOException;
@@ -37,7 +37,7 @@ import java.util.regex.Pattern;
  *
  * @author : Paul Taylor
  * @author : Eric Farng
- * @version $Id: ID3v22Frame.java,v 1.34 2009/11/12 15:42:57 paultaylor Exp $
+ * @version $Id: ID3v22Frame.java 929 2010-11-17 12:36:46Z paultaylor $
  */
 public class ID3v22Frame extends AbstractID3v2Frame
 {
@@ -52,6 +52,21 @@ public class ID3v22Frame extends AbstractID3v2Frame
 
     }
 
+    protected int getFrameIdSize()
+    {
+        return FRAME_ID_SIZE;
+    }
+
+    protected int getFrameSizeSize()
+    {
+        return FRAME_SIZE_SIZE;
+    }
+
+    protected int getFrameHeaderSize()
+    {
+        return FRAME_HEADER_SIZE;
+    }
+
     /**
      * Creates a new ID3v22 Frame with given body
      *
@@ -62,6 +77,34 @@ public class ID3v22Frame extends AbstractID3v2Frame
         super(body);
     }
 
+     /**
+     * Compare for equality
+     * To be deemed equal obj must be a IDv23Frame with the same identifier
+     * and the same flags.
+     * containing the same body,datatype list ectera.
+     * equals() method is made up from all the various components
+     *
+     * @param obj
+     * @return if true if this object is equivalent to obj
+     */
+    public boolean equals(Object obj)
+    {
+        if ( this == obj ) return true;
+
+        if (!(obj instanceof ID3v22Frame))
+        {
+            return false;
+        }
+        ID3v22Frame that = (ID3v22Frame) obj;
+
+
+        return
+              EqualsUtil.areEqual(this.statusFlags, that.statusFlags) &&
+              EqualsUtil.areEqual(this.encodingFlags, that.encodingFlags) &&
+              super.equals(that);
+
+    }
+
     /**
      * Creates a new ID3v22 Frame of type identifier.
      * <p/>
@@ -85,10 +128,14 @@ public class ID3v22Frame extends AbstractID3v2Frame
             {
                 //Do not convert
             }
+            else if(bodyIdentifier.equals("CRM"))
+            {
+                //Do not convert.
+                //TODO we don't have a way of converting this to v23 which is why its not in the ForceMap
+            }
             //TODO Improve messy fix for datetime
             //TODO need to check in case v22 body does exist before using V23 body(e.g PIC)
-            else
-            if ((bodyIdentifier.equals(ID3v22Frames.FRAME_ID_V2_TYER)) || (bodyIdentifier.equals(ID3v22Frames.FRAME_ID_V2_TIME)))
+            else if ((bodyIdentifier.equals(ID3v22Frames.FRAME_ID_V2_TYER)) || (bodyIdentifier.equals(ID3v22Frames.FRAME_ID_V2_TIME)))
             {
                 bodyIdentifier = ID3v24Frames.FRAME_ID_YEAR;
             }
@@ -228,7 +275,7 @@ public class ID3v22Frame extends AbstractID3v2Frame
      * @param loggingFilename
      * @throws org.jaudiotagger.tag.InvalidFrameException
      */
-    public ID3v22Frame(ByteBuffer byteBuffer, String loggingFilename) throws InvalidFrameException
+    public ID3v22Frame(ByteBuffer byteBuffer, String loggingFilename) throws InvalidFrameException, InvalidDataTypeException
     {
         setLoggingFilename(loggingFilename);
         read(byteBuffer);
@@ -241,7 +288,7 @@ public class ID3v22Frame extends AbstractID3v2Frame
      * @deprecated use {@link #ID3v22Frame(ByteBuffer,String)} instead
      * @throws org.jaudiotagger.tag.InvalidFrameException
      */
-    public ID3v22Frame(ByteBuffer byteBuffer) throws InvalidFrameException
+    public ID3v22Frame(ByteBuffer byteBuffer) throws InvalidFrameException, InvalidDataTypeException
     {
         this(byteBuffer, "");
     }
@@ -253,7 +300,21 @@ public class ID3v22Frame extends AbstractID3v2Frame
      */
     public int getSize()
     {
-        return frameBody.getSize() + FRAME_HEADER_SIZE;
+        return frameBody.getSize() + getFrameHeaderSize();
+    }
+
+    @Override 
+    protected boolean isPadding(byte[] buffer)
+    {
+        if(
+                (buffer[0]=='\0')&&
+                (buffer[1]=='\0')&&
+                (buffer[2]=='\0')
+           )
+        {
+            return true;
+        }
+        return false;
     }
 
     /**
@@ -262,30 +323,21 @@ public class ID3v22Frame extends AbstractID3v2Frame
      *
      * @param byteBuffer
      */
-    public void read(ByteBuffer byteBuffer) throws InvalidFrameException
+    public void read(ByteBuffer byteBuffer) throws InvalidFrameException, InvalidDataTypeException
     {
-        byte[] buffer = new byte[FRAME_ID_SIZE];
+        String identifier = readIdentifier(byteBuffer);
 
-        if (byteBuffer.position() + FRAME_HEADER_SIZE >= byteBuffer.limit())
-        {
-            logger.warning("No space to find another frame:");
-            throw new InvalidFrameException(" No space to find another frame");
-        }
-
-        // Read the FrameID Identifier
-        byteBuffer.get(buffer, 0, FRAME_ID_SIZE);
-        identifier = new String(buffer);
-        logger.info("Read Frame from file identifier is:" + identifier);
+        byte[] buffer = new byte[getFrameSizeSize()];
 
         // Is this a valid identifier?
         if (!isValidID3v2FrameIdentifier(identifier))
         {
             logger.info("Invalid identifier:" + identifier);
-            byteBuffer.position(byteBuffer.position() - (FRAME_ID_SIZE - 1));
-            throw new InvalidFrameIdentifierException(identifier + " is not a valid ID3v2.20 frame");
+            byteBuffer.position(byteBuffer.position() - (getFrameIdSize() - 1));
+            throw new InvalidFrameIdentifierException(getLoggingFilename() + ":" + identifier + ":is not a valid ID3v2.20 frame");
         }
         //Read Frame Size (same size as Frame Id so reuse buffer)
-        byteBuffer.get(buffer, 0, FRAME_SIZE_SIZE);
+        byteBuffer.get(buffer, 0, getFrameSizeSize());
         frameSize = decodeSize(buffer);
         if (frameSize < 0)
         {
@@ -371,7 +423,7 @@ public class ID3v22Frame extends AbstractID3v2Frame
         logger.info("Write Frame to Buffer" + getIdentifier());
         //This is where we will write header, move position to where we can
         //write body
-        ByteBuffer headerBuffer = ByteBuffer.allocate(FRAME_HEADER_SIZE);
+        ByteBuffer headerBuffer = ByteBuffer.allocate(getFrameHeaderSize());
 
         //Write Frame Body Data
         ByteArrayOutputStream bodyOutputStream = new ByteArrayOutputStream();
@@ -379,7 +431,7 @@ public class ID3v22Frame extends AbstractID3v2Frame
 
         //Write Frame Header
         //Write Frame ID must adjust can only be 3 bytes long
-        headerBuffer.put(Utils.getDefaultBytes(getIdentifier(), "ISO-8859-1"), 0, FRAME_ID_SIZE);
+        headerBuffer.put(Utils.getDefaultBytes(getIdentifier(), "ISO-8859-1"), 0, getFrameIdSize());
         encodeSize(headerBuffer, frameBody.getSize());
 
         //Add header to the Byte Array Output Stream
@@ -450,4 +502,21 @@ public class ID3v22Frame extends AbstractID3v2Frame
     {
         return ID3v22Frames.getInstanceOf().isBinary(getId());
     }
+
+     /**
+     * Sets the charset encoding used by the field.
+     *
+     * @param encoding charset.
+     */
+    public void setEncoding(String encoding)
+    {
+        Integer encodingId = TextEncoding.getInstanceOf().getIdForValue(encoding);
+        if(encoding!=null)
+        {
+            if(encodingId <2)
+            {
+                this.getBody().setTextEncoding(encodingId.byteValue());
+            }
+        }
+    }
 }
