@@ -26,7 +26,7 @@ import java.util.EnumMap;
  * ID3v22Tag
  *
  * @author Paul Taylor
- * @version $Id: ID3v22Frames.java,v 1.17 2009/11/12 12:22:34 paultaylor Exp $
+ * @version $Id: ID3v22Frames.java 929 2010-11-17 12:36:46Z paultaylor $
  */
 public class ID3v22Frames extends ID3Frames
 {
@@ -131,6 +131,7 @@ public class ID3v22Frames extends ID3Frames
         supportedFrames.add(FRAME_ID_V2_BPM);
         supportedFrames.add(FRAME_ID_V2_COMMENT);
         supportedFrames.add(FRAME_ID_V2_COMPOSER);
+        supportedFrames.add(FRAME_ID_V2_ENCRYPTED_FRAME);
         supportedFrames.add(FRAME_ID_V2_CONDUCTOR);
         supportedFrames.add(FRAME_ID_V2_CONTENT_GROUP_DESC);
         supportedFrames.add(FRAME_ID_V2_COPYRIGHTINFO);
@@ -301,56 +302,83 @@ public class ID3v22Frames extends ID3Frames
         multipleFrames.add(FRAME_ID_V2_USER_DEFINED_URL);
         multipleFrames.add(FRAME_ID_V2_COMMENT);
         multipleFrames.add(FRAME_ID_V2_UNSYNC_LYRICS);
-
-        //TODO this is a hack to allow multiple TRCK and TPOS frame sto be created then subsequently merged
-        multipleFrames.add(FRAME_ID_V2_TRACK);
-        multipleFrames.add(FRAME_ID_V2_SET);
+        multipleFrames.add(FRAME_ID_V2_GENERAL_ENCAPS_OBJECT);
 
         //Mapping generic key to id3v22 key
-        tagFieldToId3.put(FieldKey.ARTIST, ID3v22FieldKey.ARTIST);
         tagFieldToId3.put(FieldKey.ALBUM, ID3v22FieldKey.ALBUM);
-        tagFieldToId3.put(FieldKey.TITLE, ID3v22FieldKey.TITLE);
-        tagFieldToId3.put(FieldKey.TRACK, ID3v22FieldKey.TRACK);
-        tagFieldToId3.put(FieldKey.TRACK_TOTAL, ID3v22FieldKey.TRACK_TOTAL);
-        tagFieldToId3.put(FieldKey.YEAR, ID3v22FieldKey.YEAR);
-        tagFieldToId3.put(FieldKey.GENRE, ID3v22FieldKey.GENRE);
-        tagFieldToId3.put(FieldKey.COMMENT, ID3v22FieldKey.COMMENT);
         tagFieldToId3.put(FieldKey.ALBUM_ARTIST, ID3v22FieldKey.ALBUM_ARTIST);
-        tagFieldToId3.put(FieldKey.COMPOSER, ID3v22FieldKey.COMPOSER);
-        tagFieldToId3.put(FieldKey.GROUPING, ID3v22FieldKey.GROUPING);
-        tagFieldToId3.put(FieldKey.DISC_NO, ID3v22FieldKey.DISC_NO);
-        tagFieldToId3.put(FieldKey.DISC_TOTAL, ID3v22FieldKey.DISC_NO);
-        tagFieldToId3.put(FieldKey.BPM, ID3v22FieldKey.BPM);
-        tagFieldToId3.put(FieldKey.ENCODER, ID3v22FieldKey.ENCODER);
-        tagFieldToId3.put(FieldKey.MOOD, ID3v22FieldKey.MOOD);
-        tagFieldToId3.put(FieldKey.MUSICBRAINZ_ARTISTID, ID3v22FieldKey.MUSICBRAINZ_ARTISTID);
-        tagFieldToId3.put(FieldKey.MUSICBRAINZ_RELEASEID, ID3v22FieldKey.MUSICBRAINZ_RELEASEID);
-        tagFieldToId3.put(FieldKey.MUSICBRAINZ_RELEASEARTISTID, ID3v22FieldKey.MUSICBRAINZ_RELEASEARTISTID);
-        tagFieldToId3.put(FieldKey.MUSICBRAINZ_TRACK_ID, ID3v22FieldKey.MUSICBRAINZ_TRACK_ID);
-        tagFieldToId3.put(FieldKey.MUSICBRAINZ_DISC_ID, ID3v22FieldKey.MUSICBRAINZ_DISC_ID);
-        tagFieldToId3.put(FieldKey.MUSICIP_ID, ID3v22FieldKey.MUSICIP_ID);
-        tagFieldToId3.put(FieldKey.AMAZON_ID, ID3v22FieldKey.AMAZON_ID);
-        tagFieldToId3.put(FieldKey.MUSICBRAINZ_RELEASE_STATUS, ID3v22FieldKey.MUSICBRAINZ_RELEASE_STATUS);
-        tagFieldToId3.put(FieldKey.MUSICBRAINZ_RELEASE_TYPE, ID3v22FieldKey.MUSICBRAINZ_RELEASE_TYPE);
-        tagFieldToId3.put(FieldKey.MUSICBRAINZ_RELEASE_COUNTRY, ID3v22FieldKey.MUSICBRAINZ_RELEASE_COUNTRY);
-        tagFieldToId3.put(FieldKey.LYRICS, ID3v22FieldKey.LYRICS);
-        tagFieldToId3.put(FieldKey.IS_COMPILATION, ID3v22FieldKey.IS_COMPILATION);
-        tagFieldToId3.put(FieldKey.ARTIST_SORT, ID3v22FieldKey.ARTIST_SORT);
         tagFieldToId3.put(FieldKey.ALBUM_ARTIST_SORT, ID3v22FieldKey.ALBUM_ARTIST_SORT);
         tagFieldToId3.put(FieldKey.ALBUM_SORT, ID3v22FieldKey.ALBUM_SORT);
-        tagFieldToId3.put(FieldKey.TITLE_SORT, ID3v22FieldKey.TITLE_SORT);
+        tagFieldToId3.put(FieldKey.AMAZON_ID, ID3v22FieldKey.AMAZON_ID);
+        tagFieldToId3.put(FieldKey.ARTIST, ID3v22FieldKey.ARTIST);
+        tagFieldToId3.put(FieldKey.ARTIST_SORT, ID3v22FieldKey.ARTIST_SORT);
+        tagFieldToId3.put(FieldKey.BARCODE, ID3v22FieldKey.BARCODE);
+        tagFieldToId3.put(FieldKey.BPM, ID3v22FieldKey.BPM);
+        tagFieldToId3.put(FieldKey.CATALOG_NO, ID3v22FieldKey.CATALOG_NO);
+        tagFieldToId3.put(FieldKey.COMMENT, ID3v22FieldKey.COMMENT);
+        tagFieldToId3.put(FieldKey.COMPOSER, ID3v22FieldKey.COMPOSER);
         tagFieldToId3.put(FieldKey.COMPOSER_SORT, ID3v22FieldKey.COMPOSER_SORT);
+        tagFieldToId3.put(FieldKey.CONDUCTOR, ID3v22FieldKey.CONDUCTOR);
         tagFieldToId3.put(FieldKey.COVER_ART, ID3v22FieldKey.COVER_ART);
+        tagFieldToId3.put(FieldKey.CUSTOM1, ID3v22FieldKey.CUSTOM1);
+        tagFieldToId3.put(FieldKey.CUSTOM2, ID3v22FieldKey.CUSTOM2);
+        tagFieldToId3.put(FieldKey.CUSTOM3, ID3v22FieldKey.CUSTOM3);
+        tagFieldToId3.put(FieldKey.CUSTOM4, ID3v22FieldKey.CUSTOM4);
+        tagFieldToId3.put(FieldKey.CUSTOM5, ID3v22FieldKey.CUSTOM5);
+        tagFieldToId3.put(FieldKey.DISC_NO, ID3v22FieldKey.DISC_NO);
+        tagFieldToId3.put(FieldKey.DISC_TOTAL, ID3v22FieldKey.DISC_NO);
+        tagFieldToId3.put(FieldKey.ENCODER, ID3v22FieldKey.ENCODER);
+        tagFieldToId3.put(FieldKey.FBPM, ID3v22FieldKey.FBPM);
+        tagFieldToId3.put(FieldKey.GENRE, ID3v22FieldKey.GENRE);
+        tagFieldToId3.put(FieldKey.GROUPING, ID3v22FieldKey.GROUPING);
+        tagFieldToId3.put(FieldKey.ISRC, ID3v22FieldKey.ISRC);
+        tagFieldToId3.put(FieldKey.IS_COMPILATION, ID3v22FieldKey.IS_COMPILATION);
+        tagFieldToId3.put(FieldKey.KEY, ID3v22FieldKey.KEY);
+        tagFieldToId3.put(FieldKey.LANGUAGE, ID3v22FieldKey.LANGUAGE);
+        tagFieldToId3.put(FieldKey.LYRICIST, ID3v22FieldKey.LYRICIST);
+        tagFieldToId3.put(FieldKey.LYRICS, ID3v22FieldKey.LYRICS);
+        tagFieldToId3.put(FieldKey.MEDIA, ID3v22FieldKey.MEDIA);
+        tagFieldToId3.put(FieldKey.MOOD, ID3v22FieldKey.MOOD);
+        tagFieldToId3.put(FieldKey.MUSICBRAINZ_ARTISTID, ID3v22FieldKey.MUSICBRAINZ_ARTISTID);
+        tagFieldToId3.put(FieldKey.MUSICBRAINZ_DISC_ID, ID3v22FieldKey.MUSICBRAINZ_DISC_ID);
+        tagFieldToId3.put(FieldKey.MUSICBRAINZ_RELEASEARTISTID, ID3v22FieldKey.MUSICBRAINZ_RELEASEARTISTID);
+        tagFieldToId3.put(FieldKey.MUSICBRAINZ_RELEASEID, ID3v22FieldKey.MUSICBRAINZ_RELEASEID);
+        tagFieldToId3.put(FieldKey.MUSICBRAINZ_RELEASE_COUNTRY, ID3v22FieldKey.MUSICBRAINZ_RELEASE_COUNTRY);
+        tagFieldToId3.put(FieldKey.MUSICBRAINZ_RELEASE_GROUP_ID, ID3v22FieldKey.MUSICBRAINZ_RELEASE_GROUP_ID);
+        tagFieldToId3.put(FieldKey.MUSICBRAINZ_RELEASE_STATUS, ID3v22FieldKey.MUSICBRAINZ_RELEASE_STATUS);
+        tagFieldToId3.put(FieldKey.MUSICBRAINZ_RELEASE_TYPE, ID3v22FieldKey.MUSICBRAINZ_RELEASE_TYPE);
+        tagFieldToId3.put(FieldKey.MUSICBRAINZ_TRACK_ID, ID3v22FieldKey.MUSICBRAINZ_TRACK_ID);
+        tagFieldToId3.put(FieldKey.MUSICBRAINZ_WORK_ID, ID3v22FieldKey.MUSICBRAINZ_WORK_ID);
+        tagFieldToId3.put(FieldKey.MUSICIP_ID, ID3v22FieldKey.MUSICIP_ID);
+        tagFieldToId3.put(FieldKey.OCCASION, ID3v22FieldKey.OCCASION);
+        tagFieldToId3.put(FieldKey.ORIGINAL_ALBUM, ID3v22FieldKey.ORIGINAL_ALBUM);
+        tagFieldToId3.put(FieldKey.ORIGINAL_ARTIST, ID3v22FieldKey.ORIGINAL_ARTIST);
+        tagFieldToId3.put(FieldKey.ORIGINAL_LYRICIST, ID3v22FieldKey.ORIGINAL_LYRICIST);
+        tagFieldToId3.put(FieldKey.ORIGINAL_YEAR, ID3v22FieldKey.ORIGINAL_YEAR);
+        tagFieldToId3.put(FieldKey.QUALITY, ID3v22FieldKey.QUALITY);
+        tagFieldToId3.put(FieldKey.RATING, ID3v22FieldKey.RATING);
+        tagFieldToId3.put(FieldKey.RECORD_LABEL, ID3v22FieldKey.RECORD_LABEL);
+        tagFieldToId3.put(FieldKey.REMIXER, ID3v22FieldKey.REMIXER);
+        tagFieldToId3.put(FieldKey.SCRIPT, ID3v22FieldKey.SCRIPT);
+        tagFieldToId3.put(FieldKey.TAGS, ID3v22FieldKey.TAGS);
+        tagFieldToId3.put(FieldKey.TEMPO, ID3v22FieldKey.TEMPO);
+        tagFieldToId3.put(FieldKey.TITLE, ID3v22FieldKey.TITLE);
+        tagFieldToId3.put(FieldKey.TITLE_SORT, ID3v22FieldKey.TITLE_SORT);
+        tagFieldToId3.put(FieldKey.TRACK, ID3v22FieldKey.TRACK);
+        tagFieldToId3.put(FieldKey.TRACK_TOTAL, ID3v22FieldKey.TRACK_TOTAL);
         tagFieldToId3.put(FieldKey.URL_DISCOGS_ARTIST_SITE, ID3v22FieldKey.URL_DISCOGS_ARTIST_SITE);
         tagFieldToId3.put(FieldKey.URL_DISCOGS_RELEASE_SITE, ID3v22FieldKey.URL_DISCOGS_RELEASE_SITE);
-        tagFieldToId3.put(FieldKey.URL_WIKIPEDIA_ARTIST_SITE, ID3v22FieldKey.URL_WIKIPEDIA_ARTIST_SITE);
-        tagFieldToId3.put(FieldKey.URL_WIKIPEDIA_RELEASE_SITE, ID3v22FieldKey.URL_WIKIPEDIA_RELEASE_SITE);
+        tagFieldToId3.put(FieldKey.URL_LYRICS_SITE, ID3v22FieldKey.URL_LYRICS_SITE);
         tagFieldToId3.put(FieldKey.URL_OFFICIAL_ARTIST_SITE, ID3v22FieldKey.URL_OFFICIAL_ARTIST_SITE);
         tagFieldToId3.put(FieldKey.URL_OFFICIAL_RELEASE_SITE, ID3v22FieldKey.URL_OFFICIAL_RELEASE_SITE);
-        tagFieldToId3.put(FieldKey.LANGUAGE, ID3v22FieldKey.LANGUAGE);
-        tagFieldToId3.put(FieldKey.KEY, ID3v22FieldKey.KEY);
-        tagFieldToId3.put(FieldKey.URL_LYRICS_SITE, ID3v22FieldKey.URL_LYRICS_SITE);
-
+        tagFieldToId3.put(FieldKey.URL_WIKIPEDIA_ARTIST_SITE, ID3v22FieldKey.URL_WIKIPEDIA_ARTIST_SITE);
+        tagFieldToId3.put(FieldKey.URL_WIKIPEDIA_RELEASE_SITE, ID3v22FieldKey.URL_WIKIPEDIA_RELEASE_SITE);
+        tagFieldToId3.put(FieldKey.YEAR, ID3v22FieldKey.YEAR);
+        tagFieldToId3.put(FieldKey.ENGINEER, ID3v22FieldKey.ENGINEER);
+        tagFieldToId3.put(FieldKey.PRODUCER, ID3v22FieldKey.PRODUCER);
+        tagFieldToId3.put(FieldKey.MIXER, ID3v22FieldKey.MIXER);
+        tagFieldToId3.put(FieldKey.DJMIXER, ID3v22FieldKey.DJMIXER);
+        tagFieldToId3.put(FieldKey.ARRANGER, ID3v22FieldKey.ARRANGER);
     }
 
     /**
