@@ -26,12 +26,12 @@ import java.util.EnumMap;
  * ID3v24Tag
  *
  * @author Paul Taylor
- * @version $Id: ID3v24Frames.java,v 1.20 2009/11/12 12:22:35 paultaylor Exp $
+ * @version $Id: ID3v24Frames.java 929 2010-11-17 12:36:46Z paultaylor $
  */
 public class ID3v24Frames extends ID3Frames
 {
     /**
-     * Frame IDs begining with T are text frames, & with W are url frames
+     * Frame IDs beginning with T are text frames, & with W are url frames
      */
     public static final String FRAME_ID_ACCOMPANIMENT = "TPE2";
     public static final String FRAME_ID_ALBUM = "TALB";
@@ -345,10 +345,7 @@ public class ID3v24Frames extends ID3Frames
         multipleFrames.add(FRAME_ID_UNIQUE_FILE_ID);
         multipleFrames.add(FRAME_ID_UNSYNC_LYRICS);
         multipleFrames.add(FRAME_ID_POPULARIMETER);
-
-        //TODO this is a hack to allow multiple TRCK and TPOS frames to be created then subsequently merged
-        multipleFrames.add(FRAME_ID_TRACK);
-        multipleFrames.add(FRAME_ID_SET);
+        multipleFrames.add(FRAME_ID_GENERAL_ENCAPS_OBJECT);
 
         discardIfFileAlteredFrames.add(FRAME_ID_EVENT_TIMING_CODES);
         discardIfFileAlteredFrames.add(FRAME_ID_MPEG_LOCATION_LOOKUP_TABLE);
@@ -359,50 +356,80 @@ public class ID3v24Frames extends ID3Frames
         discardIfFileAlteredFrames.add(FRAME_ID_ENCODEDBY);
         discardIfFileAlteredFrames.add(FRAME_ID_LENGTH);
 
-        tagFieldToId3.put(FieldKey.ARTIST, ID3v24FieldKey.ARTIST);
         tagFieldToId3.put(FieldKey.ALBUM, ID3v24FieldKey.ALBUM);
-        tagFieldToId3.put(FieldKey.TITLE, ID3v24FieldKey.TITLE);
-        tagFieldToId3.put(FieldKey.TRACK, ID3v24FieldKey.TRACK);
-        tagFieldToId3.put(FieldKey.TRACK_TOTAL, ID3v24FieldKey.TRACK_TOTAL);
-        tagFieldToId3.put(FieldKey.YEAR, ID3v24FieldKey.YEAR);
-        tagFieldToId3.put(FieldKey.GENRE, ID3v24FieldKey.GENRE);
-        tagFieldToId3.put(FieldKey.COMMENT, ID3v24FieldKey.COMMENT);
         tagFieldToId3.put(FieldKey.ALBUM_ARTIST, ID3v24FieldKey.ALBUM_ARTIST);
-        tagFieldToId3.put(FieldKey.COMPOSER, ID3v24FieldKey.COMPOSER);
-        tagFieldToId3.put(FieldKey.GROUPING, ID3v24FieldKey.GROUPING);
-        tagFieldToId3.put(FieldKey.DISC_NO, ID3v24FieldKey.DISC_NO);
-        tagFieldToId3.put(FieldKey.DISC_TOTAL, ID3v24FieldKey.DISC_NO);
-        tagFieldToId3.put(FieldKey.BPM, ID3v24FieldKey.BPM);
-        tagFieldToId3.put(FieldKey.ENCODER, ID3v24FieldKey.ENCODER);
-        tagFieldToId3.put(FieldKey.MUSICBRAINZ_ARTISTID, ID3v24FieldKey.MUSICBRAINZ_ARTISTID);
-        tagFieldToId3.put(FieldKey.MUSICBRAINZ_RELEASEID, ID3v24FieldKey.MUSICBRAINZ_RELEASEID);
-        tagFieldToId3.put(FieldKey.MUSICBRAINZ_RELEASEARTISTID, ID3v24FieldKey.MUSICBRAINZ_RELEASEARTISTID);
-        tagFieldToId3.put(FieldKey.MUSICBRAINZ_TRACK_ID, ID3v24FieldKey.MUSICBRAINZ_TRACK_ID);
-        tagFieldToId3.put(FieldKey.MUSICBRAINZ_DISC_ID, ID3v24FieldKey.MUSICBRAINZ_DISC_ID);
-        tagFieldToId3.put(FieldKey.MUSICIP_ID, ID3v24FieldKey.MUSICIP_ID);
-        tagFieldToId3.put(FieldKey.AMAZON_ID, ID3v24FieldKey.AMAZON_ID);
-        tagFieldToId3.put(FieldKey.MOOD, ID3v24FieldKey.MOOD);
-        tagFieldToId3.put(FieldKey.MUSICBRAINZ_RELEASE_STATUS, ID3v24FieldKey.MUSICBRAINZ_RELEASE_STATUS);
-        tagFieldToId3.put(FieldKey.MUSICBRAINZ_RELEASE_TYPE, ID3v24FieldKey.MUSICBRAINZ_RELEASE_TYPE);
-        tagFieldToId3.put(FieldKey.MUSICBRAINZ_RELEASE_COUNTRY, ID3v24FieldKey.MUSICBRAINZ_RELEASE_COUNTRY);
-        tagFieldToId3.put(FieldKey.LYRICS, ID3v24FieldKey.LYRICS);
-        tagFieldToId3.put(FieldKey.IS_COMPILATION, ID3v24FieldKey.IS_COMPILATION);
-        tagFieldToId3.put(FieldKey.ARTIST_SORT, ID3v24FieldKey.ARTIST_SORT);
         tagFieldToId3.put(FieldKey.ALBUM_ARTIST_SORT, ID3v24FieldKey.ALBUM_ARTIST_SORT);
         tagFieldToId3.put(FieldKey.ALBUM_SORT, ID3v24FieldKey.ALBUM_SORT);
-        tagFieldToId3.put(FieldKey.TITLE_SORT, ID3v24FieldKey.TITLE_SORT);
+        tagFieldToId3.put(FieldKey.AMAZON_ID, ID3v24FieldKey.AMAZON_ID);
+        tagFieldToId3.put(FieldKey.ARTIST, ID3v24FieldKey.ARTIST);
+        tagFieldToId3.put(FieldKey.ARTIST_SORT, ID3v24FieldKey.ARTIST_SORT);
+        tagFieldToId3.put(FieldKey.BARCODE, ID3v24FieldKey.BARCODE);
+        tagFieldToId3.put(FieldKey.BPM, ID3v24FieldKey.BPM);
+        tagFieldToId3.put(FieldKey.CATALOG_NO, ID3v24FieldKey.CATALOG_NO);
+        tagFieldToId3.put(FieldKey.COMMENT, ID3v24FieldKey.COMMENT);
+        tagFieldToId3.put(FieldKey.COMPOSER, ID3v24FieldKey.COMPOSER);
         tagFieldToId3.put(FieldKey.COMPOSER_SORT, ID3v24FieldKey.COMPOSER_SORT);
+        tagFieldToId3.put(FieldKey.CONDUCTOR, ID3v24FieldKey.CONDUCTOR);
         tagFieldToId3.put(FieldKey.COVER_ART, ID3v24FieldKey.COVER_ART);
+        tagFieldToId3.put(FieldKey.CUSTOM1, ID3v24FieldKey.CUSTOM1);
+        tagFieldToId3.put(FieldKey.CUSTOM2, ID3v24FieldKey.CUSTOM2);
+        tagFieldToId3.put(FieldKey.CUSTOM3, ID3v24FieldKey.CUSTOM3);
+        tagFieldToId3.put(FieldKey.CUSTOM4, ID3v24FieldKey.CUSTOM4);
+        tagFieldToId3.put(FieldKey.CUSTOM5, ID3v24FieldKey.CUSTOM5);
+        tagFieldToId3.put(FieldKey.DISC_NO, ID3v24FieldKey.DISC_NO);
+        tagFieldToId3.put(FieldKey.DISC_TOTAL, ID3v24FieldKey.DISC_NO);
+        tagFieldToId3.put(FieldKey.ENCODER, ID3v24FieldKey.ENCODER);
+        tagFieldToId3.put(FieldKey.FBPM, ID3v24FieldKey.FBPM);
+        tagFieldToId3.put(FieldKey.GENRE, ID3v24FieldKey.GENRE);
+        tagFieldToId3.put(FieldKey.GROUPING, ID3v24FieldKey.GROUPING);
+        tagFieldToId3.put(FieldKey.ISRC, ID3v24FieldKey.ISRC);
+        tagFieldToId3.put(FieldKey.IS_COMPILATION, ID3v24FieldKey.IS_COMPILATION);
+        tagFieldToId3.put(FieldKey.KEY, ID3v24FieldKey.KEY);
+        tagFieldToId3.put(FieldKey.LANGUAGE, ID3v24FieldKey.LANGUAGE);
+        tagFieldToId3.put(FieldKey.LYRICIST, ID3v24FieldKey.LYRICIST);
+        tagFieldToId3.put(FieldKey.LYRICS, ID3v24FieldKey.LYRICS);
+        tagFieldToId3.put(FieldKey.MEDIA, ID3v24FieldKey.MEDIA);
+        tagFieldToId3.put(FieldKey.MOOD, ID3v24FieldKey.MOOD);
+        tagFieldToId3.put(FieldKey.MUSICBRAINZ_ARTISTID, ID3v24FieldKey.MUSICBRAINZ_ARTISTID);
+        tagFieldToId3.put(FieldKey.MUSICBRAINZ_DISC_ID, ID3v24FieldKey.MUSICBRAINZ_DISC_ID);
+        tagFieldToId3.put(FieldKey.MUSICBRAINZ_RELEASEARTISTID, ID3v24FieldKey.MUSICBRAINZ_RELEASEARTISTID);
+        tagFieldToId3.put(FieldKey.MUSICBRAINZ_RELEASEID, ID3v24FieldKey.MUSICBRAINZ_RELEASEID);
+        tagFieldToId3.put(FieldKey.MUSICBRAINZ_RELEASE_COUNTRY, ID3v24FieldKey.MUSICBRAINZ_RELEASE_COUNTRY);
+        tagFieldToId3.put(FieldKey.MUSICBRAINZ_RELEASE_GROUP_ID, ID3v24FieldKey.MUSICBRAINZ_RELEASE_GROUP_ID);
+        tagFieldToId3.put(FieldKey.MUSICBRAINZ_RELEASE_STATUS, ID3v24FieldKey.MUSICBRAINZ_RELEASE_STATUS);
+        tagFieldToId3.put(FieldKey.MUSICBRAINZ_RELEASE_TYPE, ID3v24FieldKey.MUSICBRAINZ_RELEASE_TYPE);
+        tagFieldToId3.put(FieldKey.MUSICBRAINZ_TRACK_ID, ID3v24FieldKey.MUSICBRAINZ_TRACK_ID);
+        tagFieldToId3.put(FieldKey.MUSICBRAINZ_WORK_ID, ID3v24FieldKey.MUSICBRAINZ_WORK_ID);
+        tagFieldToId3.put(FieldKey.MUSICIP_ID, ID3v24FieldKey.MUSICIP_ID);
+        tagFieldToId3.put(FieldKey.OCCASION, ID3v24FieldKey.OCCASION);
+        tagFieldToId3.put(FieldKey.ORIGINAL_ALBUM, ID3v24FieldKey.ORIGINAL_ALBUM);
+        tagFieldToId3.put(FieldKey.ORIGINAL_ARTIST, ID3v24FieldKey.ORIGINAL_ARTIST);
+        tagFieldToId3.put(FieldKey.ORIGINAL_LYRICIST, ID3v24FieldKey.ORIGINAL_LYRICIST);
+        tagFieldToId3.put(FieldKey.ORIGINAL_YEAR, ID3v24FieldKey.ORIGINAL_YEAR);
+        tagFieldToId3.put(FieldKey.QUALITY, ID3v24FieldKey.QUALITY);
+        tagFieldToId3.put(FieldKey.RATING, ID3v24FieldKey.RATING);
+        tagFieldToId3.put(FieldKey.RECORD_LABEL, ID3v24FieldKey.RECORD_LABEL);
+        tagFieldToId3.put(FieldKey.REMIXER, ID3v24FieldKey.REMIXER);
+        tagFieldToId3.put(FieldKey.SCRIPT, ID3v24FieldKey.SCRIPT);
+        tagFieldToId3.put(FieldKey.TAGS, ID3v24FieldKey.TAGS);
+        tagFieldToId3.put(FieldKey.TEMPO, ID3v24FieldKey.TEMPO);
+        tagFieldToId3.put(FieldKey.TITLE, ID3v24FieldKey.TITLE);
+        tagFieldToId3.put(FieldKey.TITLE_SORT, ID3v24FieldKey.TITLE_SORT);
+        tagFieldToId3.put(FieldKey.TRACK, ID3v24FieldKey.TRACK);
+        tagFieldToId3.put(FieldKey.TRACK_TOTAL, ID3v24FieldKey.TRACK_TOTAL);
         tagFieldToId3.put(FieldKey.URL_DISCOGS_ARTIST_SITE, ID3v24FieldKey.URL_DISCOGS_ARTIST_SITE);
         tagFieldToId3.put(FieldKey.URL_DISCOGS_RELEASE_SITE, ID3v24FieldKey.URL_DISCOGS_RELEASE_SITE);
-        tagFieldToId3.put(FieldKey.URL_WIKIPEDIA_ARTIST_SITE, ID3v24FieldKey.URL_WIKIPEDIA_ARTIST_SITE);
-        tagFieldToId3.put(FieldKey.URL_WIKIPEDIA_RELEASE_SITE, ID3v24FieldKey.URL_WIKIPEDIA_RELEASE_SITE);
+        tagFieldToId3.put(FieldKey.URL_LYRICS_SITE, ID3v24FieldKey.URL_LYRICS_SITE);
         tagFieldToId3.put(FieldKey.URL_OFFICIAL_ARTIST_SITE, ID3v24FieldKey.URL_OFFICIAL_ARTIST_SITE);
         tagFieldToId3.put(FieldKey.URL_OFFICIAL_RELEASE_SITE, ID3v24FieldKey.URL_OFFICIAL_RELEASE_SITE);
-        tagFieldToId3.put(FieldKey.LANGUAGE, ID3v24FieldKey.LANGUAGE);
-        tagFieldToId3.put(FieldKey.KEY, ID3v24FieldKey.KEY);
-        tagFieldToId3.put(FieldKey.URL_LYRICS_SITE, ID3v24FieldKey.URL_LYRICS_SITE);
-
+        tagFieldToId3.put(FieldKey.URL_WIKIPEDIA_ARTIST_SITE, ID3v24FieldKey.URL_WIKIPEDIA_ARTIST_SITE);
+        tagFieldToId3.put(FieldKey.URL_WIKIPEDIA_RELEASE_SITE, ID3v24FieldKey.URL_WIKIPEDIA_RELEASE_SITE);
+        tagFieldToId3.put(FieldKey.YEAR, ID3v24FieldKey.YEAR);
+        tagFieldToId3.put(FieldKey.ENGINEER, ID3v24FieldKey.ENGINEER);
+        tagFieldToId3.put(FieldKey.PRODUCER, ID3v24FieldKey.PRODUCER);
+        tagFieldToId3.put(FieldKey.MIXER, ID3v24FieldKey.MIXER);
+        tagFieldToId3.put(FieldKey.DJMIXER, ID3v24FieldKey.DJMIXER);
+        tagFieldToId3.put(FieldKey.ARRANGER, ID3v24FieldKey.ARRANGER);
     }
 
     /**
