@@ -40,7 +40,7 @@ import java.util.logging.Level;
  *
  * @author : Paul Taylor
  * @author : Eric Farng
- * @version $Id: ID3v24Tag.java,v 1.57 2009/11/12 15:42:58 paultaylor Exp $
+ * @version $Id: ID3v24Tag.java 929 2010-11-17 12:36:46Z paultaylor $
  */
 public class ID3v24Tag extends AbstractID3v2Tag
 {
@@ -306,6 +306,8 @@ public class ID3v24Tag extends AbstractID3v2Tag
     public ID3v24Tag()
     {
         frameMap = new LinkedHashMap();
+        encryptedFrameMap = new LinkedHashMap();
+                
     }
 
     /**
@@ -335,8 +337,15 @@ public class ID3v24Tag extends AbstractID3v2Tag
     {
         try
         {
-            ID3v24Frame newFrame = new ID3v24Frame(frame);
-            copyFrameIntoMap(newFrame.getIdentifier(), newFrame);
+            if (frame instanceof ID3v24Frame)
+            {
+                 copyFrameIntoMap(frame.getIdentifier(),frame);
+            }
+            else
+            {
+                ID3v24Frame newFrame = new ID3v24Frame(frame);
+                copyFrameIntoMap(newFrame.getIdentifier(), newFrame);
+            }
         }
         catch (InvalidFrameException ife)
         {
@@ -345,12 +354,12 @@ public class ID3v24Tag extends AbstractID3v2Tag
     }
 
     /*
-       * Copy framne into map, whilst accounting for multiple frame sof same type which can occur even if there were
+       * Copy frame into map, whilst accounting for multiple frames of same type which can occur even if there were
        * not frames of the dame type in the original tag
        *
-       * The frame already exists this shouldnt normally happen because frames
+       * The frame already exists this shouldn't normally happen because frames
        * that are allowed to be multiple don't call this method. Frames that
-       * arent allowed to be multiple aren't added to hashmap in first place when
+       * aren't allowed to be multiple aren't added to hashMap in first place when
        * originally added.
        *
        * We only want to allow one of the frames going forward but we try and merge
@@ -360,7 +369,7 @@ public class ID3v24Tag extends AbstractID3v2Tag
        *
        * However converting some frames from tag of one version to another may
        * mean that two different frames both get converted to one frame, this
-       * particulary applies to DateTime fields which were originally two fields
+       * particularly applies to DateTime fields which were originally two fields
        * in v2.3 but are one field in v2.4.
        */
        @Override
@@ -464,6 +473,8 @@ public class ID3v24Tag extends AbstractID3v2Tag
     {
         logger.info("Creating tag from a tag of a different version");
         frameMap = new LinkedHashMap();
+        encryptedFrameMap = new LinkedHashMap();
+
         if (mp3tag != null)
         {
             //Should use simpler copy constructor
@@ -477,6 +488,7 @@ public class ID3v24Tag extends AbstractID3v2Tag
              */
             else if (mp3tag instanceof AbstractID3v2Tag)
             {
+               this.setLoggingFilename(((AbstractID3v2Tag)mp3tag).getLoggingFilename());
                 copyPrimitives((AbstractID3v2Tag) mp3tag);
                 copyFrames((AbstractID3v2Tag) mp3tag);
             }
@@ -587,6 +599,8 @@ public class ID3v24Tag extends AbstractID3v2Tag
     public ID3v24Tag(ByteBuffer buffer, String loggingFilename) throws TagException
     {
         frameMap = new LinkedHashMap();
+        encryptedFrameMap = new LinkedHashMap();
+
         setLoggingFilename(loggingFilename);
         this.read(buffer);
     }
@@ -870,6 +884,8 @@ public class ID3v24Tag extends AbstractID3v2Tag
         //Now start looking for frames
         ID3v24Frame next;
         frameMap = new LinkedHashMap();
+        encryptedFrameMap = new LinkedHashMap();
+
         //Read the size from the Tag Header
         this.fileReadSize = size;
         // Read the frames until got to upto the size as specified in header
@@ -885,6 +901,12 @@ public class ID3v24Tag extends AbstractID3v2Tag
                 id = next.getIdentifier();
                 loadFrameIntoMap(id, next);
             }
+            //Found Padding, no more frames
+            catch (PaddingException ex)
+            {
+                logger.config(getLoggingFilename() + ":Found padding starting at:" + byteBuffer.position());
+                break;
+            }
             //Found Empty Frame
             catch (EmptyFrameException ex)
             {
@@ -894,18 +916,26 @@ public class ID3v24Tag extends AbstractID3v2Tag
             catch (InvalidFrameIdentifierException ifie)
             {
                 logger.info(getLoggingFilename() + ":" + "Invalid Frame Identifier:" + ifie.getMessage());
-                this.invalidFrameBytes++;
-                //Dont try and find any more frames
+                this.invalidFrames++;
+                //Don't try and find any more frames
                 break;
             }
             //Problem trying to find frame
             catch (InvalidFrameException ife)
             {
                 logger.warning(getLoggingFilename() + ":" + "Invalid Frame:" + ife.getMessage());
-                this.invalidFrameBytes++;
-                //Dont try and find any more frames
+                this.invalidFrames++;
+                //Don't try and find any more frames
                 break;
             }
+            //Failed reading frame but may just have invalid data but correct length so lets carry on
+            //in case we can read the next frame
+            catch(InvalidDataTypeException idete)
+            {
+                logger.warning(getLoggingFilename() + ":Corrupt Frame:" + idete.getMessage());
+                this.invalidFrames++;
+                continue;
+            }
         }
     }
 
@@ -1061,7 +1091,8 @@ public class ID3v24Tag extends AbstractID3v2Tag
     @Override
     public void write(File file, long audioStartLocation) throws IOException
     {
-        logger.info("Writing tag to file");
+        setLoggingFilename(file.getName());
+        logger.info("Writing tag to file:"+getLoggingFilename());
 
         //Write Body Buffer
         byte[] bodyByteBuffer = writeFramesToBuffer().toByteArray();
@@ -1136,41 +1167,6 @@ public class ID3v24Tag extends AbstractID3v2Tag
         return unsynchronization;
     }
 
-    protected String getArtistId()
-    {
-        return ID3v24Frames.FRAME_ID_ARTIST;
-    }
-
-    protected String getAlbumId()
-    {
-        return ID3v24Frames.FRAME_ID_ALBUM;
-    }
-
-    protected String getTitleId()
-    {
-        return ID3v24Frames.FRAME_ID_TITLE;
-    }
-
-    protected String getTrackId()
-    {
-        return ID3v24Frames.FRAME_ID_TRACK;
-    }
-
-    protected String getYearId()
-    {
-        return ID3v24Frames.FRAME_ID_YEAR;
-    }
-
-    protected String getCommentId()
-    {
-        return ID3v24Frames.FRAME_ID_COMMENT;
-    }
-
-    protected String getGenreId()
-    {
-        return ID3v24Frames.FRAME_ID_GENRE;
-    }
-
     /**
      * Create a new frame with the specified frameid
      *
@@ -1195,7 +1191,7 @@ public class ID3v24Tag extends AbstractID3v2Tag
      * @throws KeyNotFoundException
      * @throws FieldDataInvalidException
      */
-    public TagField createTagField(ID3v24FieldKey id3Key, String value) throws KeyNotFoundException, FieldDataInvalidException
+    public TagField createField(ID3v24FieldKey id3Key, String value) throws KeyNotFoundException, FieldDataInvalidException
     {
         if (id3Key == null)
         {
@@ -1241,7 +1237,7 @@ public class ID3v24Tag extends AbstractID3v2Tag
         }
         else
         {
-            return super.doGetFirst(frameAndSubId);
+            return super.doGetValueAtIndex(frameAndSubId, 0);
         }
     }
 
@@ -1252,7 +1248,7 @@ public class ID3v24Tag extends AbstractID3v2Tag
      * @param id3v24FieldKey
      * @throws org.jaudiotagger.tag.KeyNotFoundException
      */
-    public void deleteTagField(ID3v24FieldKey id3v24FieldKey) throws KeyNotFoundException
+    public void deleteField(ID3v24FieldKey id3v24FieldKey) throws KeyNotFoundException
     {
         if (id3v24FieldKey == null)
         {
@@ -1261,13 +1257,21 @@ public class ID3v24Tag extends AbstractID3v2Tag
         super.doDeleteTagField(new FrameAndSubId(id3v24FieldKey.getFrameId(), id3v24FieldKey.getSubId()));
     }
 
+    /**
+     * Delete fields with this (frame) id
+     * @param id
+     */
+    public void deleteField(String id)
+    {
+        super.doDeleteTagField(new FrameAndSubId(id,null));
+    }
 
     protected FrameAndSubId getFrameAndSubIdFromGenericKey(FieldKey genericKey)
     {
         ID3v24FieldKey id3v24FieldKey = ID3v24Frames.getInstanceOf().getId3KeyFromGenericKey(genericKey);
         if (id3v24FieldKey == null)
         {
-            throw new KeyNotFoundException();
+            throw new KeyNotFoundException("Unable to find key for "+genericKey.name());
         }
         return new FrameAndSubId(id3v24FieldKey.getFrameId(), id3v24FieldKey.getSubId());
     }
