@@ -29,7 +29,7 @@ import java.nio.ByteBuffer;
  *
  * @author : Paul Taylor
  * @author : Eric Farng
- * @version $Id: FrameBodyRVAD.java,v 1.18 2009/11/12 13:25:22 paultaylor Exp $
+ * @version $Id: FrameBodyRVAD.java 857 2009-12-03 11:21:11Z paultaylor $
  */
 public class FrameBodyRVAD extends AbstractID3v2FrameBody implements ID3v23FrameBody
 {
@@ -80,20 +80,6 @@ public class FrameBodyRVAD extends AbstractID3v2FrameBody implements ID3v23Frame
         return ID3v23Frames.FRAME_ID_V3_RELATIVE_VOLUME_ADJUSTMENT;
     }
 
-    /**
-     * This method is not yet supported.
-     *
-     * @throws java.lang.UnsupportedOperationException
-     *          This method is not yet
-     *          supported
-     * @todo Implement this java.lang.Object method
-     */
-    public boolean equals(Object obj)
-    {
-        throw new java.lang.UnsupportedOperationException("Method equals() not yet implemented.");
-    }
-
-
     /**
      * Setup the Object List. A byte Array which will be read upto frame size
      * bytes.
