@@ -9,7 +9,7 @@ package org.jaudiotagger.tag.id3.valuepair;
 public enum ID3V2ExtendedGenreTypes
 {
     RX("Remix"),
-    CV("Cover");
+    CR("Cover");
 
     private String description;
 
