@@ -1,7 +1,7 @@
 /**
  * @author : Paul Taylor
  * <p/>
- * Version @version:$Id: Languages.java,v 1.4 2009/09/03 09:19:11 paultaylor Exp $
+ * Version @version:$Id: Languages.java 929 2010-11-17 12:36:46Z paultaylor $
  * <p/>
  * Jaudiotagger Copyright (C)2004,2005
  * <p/>
@@ -27,6 +27,9 @@ import org.jaudiotagger.tag.datatype.AbstractStringStringValuePair;
 public class Languages extends AbstractStringStringValuePair
 {
     public static final String DEFAULT_ID = "eng";
+    public static final String MEDIA_MONKEY_ID = "XXX";
+    public static final String WINAMP_ID = "\0\0\0";
+
     public static final String DEFAULT_VALUE = "English";
 
     //The number of bytes used to hold the language field size
