@@ -3,12 +3,13 @@ package org.jaudiotagger.issues;
 import org.jaudiotagger.AbstractTestCase;
 import org.jaudiotagger.audio.AudioFile;
 import org.jaudiotagger.audio.AudioFileIO;
-import org.jaudiotagger.audio.asf.tag.AsfTagCoverField;
+import org.jaudiotagger.tag.asf.AsfTagCoverField;
 import org.jaudiotagger.tag.FieldKey;
 import org.jaudiotagger.tag.Tag;
 import org.jaudiotagger.tag.datatype.Artwork;
 import org.jaudiotagger.tag.id3.ID3v22Tag;
 import org.jaudiotagger.tag.id3.ID3v24Tag;
+import org.jaudiotagger.tag.id3.valuepair.ImageFormats;
 
 import java.io.File;
 
@@ -37,6 +38,8 @@ public class Issue245Test extends AbstractTestCase
 
             //Now addField the image
             Artwork newartwork = Artwork.createArtworkFromFile(new File("testdata", "coverart.png"));
+            assertTrue(ImageFormats.isPortableFormat(newartwork.getBinaryData()));
+
             newartwork.setPictureType(5);
             tag.setField(newartwork);
             af.commit();
@@ -88,6 +91,8 @@ public class Issue245Test extends AbstractTestCase
 
                     //Now addField the image
                     Artwork newartwork = Artwork.createArtworkFromFile(new File("testdata", "coverart.png"));
+                    assertTrue(ImageFormats.isPortableFormat(newartwork.getBinaryData()));
+
                     newartwork.setPictureType(11);
                     tag.setField(newartwork);
                     af.commit();
@@ -139,6 +144,8 @@ public class Issue245Test extends AbstractTestCase
 
                     //Now addField the image
                     Artwork newartwork = Artwork.createArtworkFromFile(new File("testdata", "coverart.png"));
+                    assertTrue(ImageFormats.isPortableFormat(newartwork.getBinaryData()));
+
                     newartwork.setPictureType(5);
                     tag.setField(newartwork);
                     af.commit();
@@ -190,8 +197,10 @@ public class Issue245Test extends AbstractTestCase
             assertEquals("image/png", artwork.getMimeType());
             assertNotNull(artwork.getImage());
             assertEquals(200, artwork.getImage().getWidth());
+
             //Now replace the image
             Artwork newartwork = Artwork.createArtworkFromFile(new File("testdata", "coverart.png"));
+            assertTrue(ImageFormats.isPortableFormat(newartwork.getBinaryData()));
             tag.setField(newartwork);
             af.commit();
             af = AudioFileIO.read(testFile);
@@ -229,7 +238,7 @@ public class Issue245Test extends AbstractTestCase
         Exception exceptionCaught = null;
         try
         {
-            testFile = AbstractTestCase.copyAudioToTmp("test.flac");
+            testFile = AbstractTestCase.copyAudioToTmp("test.flac",new File("testwriteartwork.flac"));
 
             //Read File okay
             AudioFile af = AudioFileIO.read(testFile);
@@ -244,6 +253,8 @@ public class Issue245Test extends AbstractTestCase
             assertEquals(3,artwork.getPictureType());
             //Now replace the image
             Artwork newartwork = Artwork.createArtworkFromFile(new File("testdata", "coverart.png"));
+            assertTrue(ImageFormats.isPortableFormat(newartwork.getBinaryData()));
+
             newartwork.setDescription("freddy");
             newartwork.setPictureType(7);
             tag.setField(newartwork);
@@ -300,6 +311,8 @@ public class Issue245Test extends AbstractTestCase
             assertEquals(3,artwork.getPictureType());
             //Now replace the image
             Artwork newartwork = Artwork.createArtworkFromFile(new File("testdata", "coverart.png"));
+            assertTrue(ImageFormats.isPortableFormat(newartwork.getBinaryData()));
+
             newartwork.setDescription("freddy");
             newartwork.setPictureType(8);
             tag.setField(newartwork);
@@ -356,6 +369,8 @@ public class Issue245Test extends AbstractTestCase
 
             //Now replace the image
             Artwork newartwork = Artwork.createArtworkFromFile(new File("testdata", "coverart.png"));
+            assertTrue(ImageFormats.isPortableFormat(newartwork.getBinaryData()));
+
             tag.setField(newartwork);
             af.commit();
             af = AudioFileIO.read(testFile);
@@ -418,6 +433,8 @@ public class Issue245Test extends AbstractTestCase
             //Now try and addField image
             AudioFile af = AudioFileIO.read(testFile);
             Artwork newartwork = Artwork.createArtworkFromFile(new File("testdata", "coverart.png"));
+            assertTrue(ImageFormats.isPortableFormat(newartwork.getBinaryData()));
+
             Tag tag = af.getTag();
             tag.setField(newartwork);
 
@@ -480,6 +497,8 @@ public class Issue245Test extends AbstractTestCase
             //Now try and addField image
             AudioFile af = AudioFileIO.read(testFile);
             Artwork newartwork = Artwork.createArtworkFromFile(new File("testdata", "coverart.png"));
+            assertTrue(ImageFormats.isPortableFormat(newartwork.getBinaryData()));
+
             Tag tag = af.getTag();
             tag.setField(newartwork);
 
