@@ -29,7 +29,7 @@ public class ID3v1TagTest extends TestCase
     public static final String YEAR = "1971";
 
     /**
-     * Provides an initilised object to be used in other tests
+     * Provides an initialised object to be used in other tests
      * to prevent code duplication
      *
      * @return ID3v1Tag
@@ -107,7 +107,7 @@ public class ID3v1TagTest extends TestCase
     // Tests
     /////////////////////////////////////////////////////////////////////////
 
-    public void testCreateID3v11Tag()
+    public void testCreateID3v1Tag()
     {
         ID3v1Tag v1Tag = new ID3v1Tag();
         v1Tag.setArtist(ID3v1TagTest.ARTIST);
@@ -166,6 +166,14 @@ public class ID3v1TagTest extends TestCase
         assertEquals(ID3v1TagTest.GENRE_VAL, tag.getFirst(FieldKey.GENRE));
         assertEquals(ID3v1TagTest.YEAR, tag.getFirst(FieldKey.YEAR));
 
+        tag.setField(FieldKey.TRACK,"3");
+        mp3File.save();
+        mp3File = new MP3File(testFile);
+        tag = mp3File.getID3v1Tag();
+        assertEquals("", tag.getFirst(FieldKey.TRACK));
+
+        
+
     }
 
 
@@ -245,7 +253,7 @@ public class ID3v1TagTest extends TestCase
         v1Tag.setField(new ID3v1TagField(FieldKey.ARTIST.name(), "artist"));
         assertEquals("artist", ((TagTextField) v1Tag.getFields(FieldKey.ARTIST).get(0)).getContent());
         assertEquals("artist", ((TagTextField) v1Tag.getFirstField(FieldKey.ARTIST.name())).getContent());
-        assertEquals("artist", ((TagTextField) (v1Tag.get(FieldKey.ARTIST.name()).get(0))).getContent());
+        assertEquals("artist", ((TagTextField) (v1Tag.getFields(FieldKey.ARTIST.name()).get(0))).getContent());
 
         v1Tag.setField(new ID3v1TagField(FieldKey.ALBUM.name(), "album"));
         assertEquals("album", ((TagTextField) v1Tag.getFields(FieldKey.ALBUM).get(0)).getContent());
