@@ -192,14 +192,22 @@ public class UnsynchronizationTest extends AbstractTestCase
             af.commit();
 
             //Now read back ok
+            long start = System.nanoTime();
             af = AudioFileIO.read(testFile2);
+            long time = System.nanoTime() - start;
+            System.out.printf("NOTSYNCED Took %6.3f ms \n", time/1e6);
+
             notSyncedTag = af.getTag();
             v23TagNotsynced = (ID3v23Tag)notSyncedTag;
             assertEquals(1,notSyncedTag.getArtworkList().size());
             artworkNotsynced = notSyncedTag.getArtworkList().get(0);
-
+            
             //Now read back ok
+            start = System.nanoTime();
             af = AudioFileIO.read(testFile);
+            time = System.nanoTime() - start;
+            System.out.printf("UNSYCNCED Took %6.3f ms \n", time/1e6);
+
             unsyncedTag = af.getTag();
             v23TagUnsynced = (ID3v23Tag)unsyncedTag;
             assertTrue(v23TagUnsynced.isUnsynchronization());
@@ -214,7 +222,7 @@ public class UnsynchronizationTest extends AbstractTestCase
             boolean matches = true;
             for(int i=0;i< artworkUnsynced.getBinaryData().length;i++)
             {
-           if((artworkUnsynced.getBinaryData()[i])!=(artworkNotsynced.getBinaryData()[i]))
+                if((artworkUnsynced.getBinaryData()[i])!=(artworkNotsynced.getBinaryData()[i]))
                 {
                     System.out.println(i+":"+ Hex.asHex(artworkNotsynced.getBinaryData()[i])+":"+Hex.asHex(artworkUnsynced.getBinaryData()[i]));
                     matches=false;
@@ -232,4 +240,6 @@ public class UnsynchronizationTest extends AbstractTestCase
         }
         assertNull(exceptionCaught);
     }
+
+
 }
