@@ -71,9 +71,9 @@ public class VorbisWriteTagTest extends AbstractTestCase
             tag.setField(tag.createField(FieldKey.DISC_NO, "1"));
             tag.setField(tag.createField(FieldKey.COMPOSER, "composer\u00A9"));
             tag.setField(tag.createField(FieldKey.ARTIST_SORT, "Sortartist\u01ff"));
-            tag.setField(tag.createField(FieldKey.LYRICS, "lyrics"));
-            tag.setField(tag.createField(FieldKey.BPM, "200"));
-            tag.setField(tag.createField(FieldKey.ALBUM_ARTIST, "Albumartist"));
+            tag.setField(FieldKey.LYRICS, "lyrics");
+            tag.setField(FieldKey.BPM, "200");
+            tag.setField(FieldKey.ALBUM_ARTIST, "Albumartist");
             tag.setField(tag.createField(FieldKey.ALBUM_ARTIST_SORT, "Sortalbumartist"));
             tag.setField(tag.createField(FieldKey.ALBUM_SORT, "Sortalbum"));
             tag.setField(tag.createField(FieldKey.GROUPING, "GROUping"));
@@ -97,7 +97,7 @@ public class VorbisWriteTagTest extends AbstractTestCase
 
 
             //Vorbis Only keys
-            tag.setField(((VorbisCommentTag) tag).createTagField(VorbisCommentFieldKey.DESCRIPTION, "description"));
+            tag.setField(((VorbisCommentTag) tag).createField(VorbisCommentFieldKey.DESCRIPTION, "description"));
 
             //tag.setField(tag.createField(FieldKey.ENCODER,"encoder"));
             tag.setVendor("encoder");
@@ -109,11 +109,12 @@ public class VorbisWriteTagTest extends AbstractTestCase
             imageFile.read(imagedata);
             char[] testdata = Base64Coder.encode(imagedata);
             String base64image = new String(testdata);
-            tag.setField(((VorbisCommentTag) tag).createTagField(VorbisCommentFieldKey.COVERART, base64image));
-            tag.setField(((VorbisCommentTag) tag).createTagField(VorbisCommentFieldKey.COVERARTMIME, "image/png"));
+            tag.setField(tag.createField(VorbisCommentFieldKey.COVERART, base64image));
+            tag.setField(tag.createField(VorbisCommentFieldKey.COVERARTMIME, "image/png"));
 
             //key not known to jaudiotagger
-            tag.setField(((VorbisCommentTag) tag).createTagField("VOLINIST", "Sarah Curtis"));
+            tag.setField("VOLINIST", "Sarah Curtis");
+            assertEquals("image/png", tag.getFirst(VorbisCommentFieldKey.COVERARTMIME));
 
             f.commit();
 
@@ -205,8 +206,8 @@ public class VorbisWriteTagTest extends AbstractTestCase
             assertEquals("description", vorbisTag.getFirst(VorbisCommentFieldKey.DESCRIPTION));
 
             //VorbisImage base64 image, and reconstruct
-            assertEquals(base64image, vorbisTag.getFirst(VorbisCommentFieldKey.COVERART));
             assertEquals("image/png", vorbisTag.getFirst(VorbisCommentFieldKey.COVERARTMIME));
+            assertEquals(base64image, vorbisTag.getFirst(VorbisCommentFieldKey.COVERART));
             BufferedImage bi = ImageIO.read(ImageIO
                     .createImageInputStream(new ByteArrayInputStream(Base64Coder.
                     decode(vorbisTag.getFirst(VorbisCommentFieldKey.COVERART).toCharArray()))));
@@ -257,8 +258,8 @@ public class VorbisWriteTagTest extends AbstractTestCase
             imageFile.read(imagedata);
             char[] testdata = Base64Coder.encode(imagedata);
             String base64image = new String(testdata);
-            tag.setField(tag.createTagField(VorbisCommentFieldKey.COVERART, base64image));
-            tag.setField(tag.createTagField(VorbisCommentFieldKey.COVERARTMIME, "image/png"));
+            tag.setField(tag.createField(VorbisCommentFieldKey.COVERART, base64image));
+            tag.setField(tag.createField(VorbisCommentFieldKey.COVERARTMIME, "image/png"));
 
             //Save
             f.commit();
@@ -315,8 +316,8 @@ public class VorbisWriteTagTest extends AbstractTestCase
             {
                 sb.append("a");
             }
-            tag.setField(tag.createTagField(VorbisCommentFieldKey.COVERART, sb.toString()));
-            tag.setField(tag.createTagField(VorbisCommentFieldKey.COVERARTMIME, "image/png"));
+            tag.setField(tag.createField(VorbisCommentFieldKey.COVERART, sb.toString()));
+            tag.setField(tag.createField(VorbisCommentFieldKey.COVERARTMIME, "image/png"));
 
             //Save
             f.commit();
@@ -492,8 +493,8 @@ public class VorbisWriteTagTest extends AbstractTestCase
             imageFile.read(imagedata);
             char[] testdata = Base64Coder.encode(imagedata);
             String base64image = new String(testdata);
-            tag.setField(tag.createTagField(VorbisCommentFieldKey.COVERART, base64image));
-            tag.setField(tag.createTagField(VorbisCommentFieldKey.COVERARTMIME, "image/png"));
+            tag.setField(tag.createField(VorbisCommentFieldKey.COVERART, base64image));
+            tag.setField(tag.createField(VorbisCommentFieldKey.COVERARTMIME, "image/png"));
 
             //Save
             f.commit();
@@ -552,8 +553,8 @@ public class VorbisWriteTagTest extends AbstractTestCase
             {
                 sb.append("a");
             }
-            tag.setField(tag.createTagField(VorbisCommentFieldKey.COVERART, sb.toString()));
-            tag.setField(tag.createTagField(VorbisCommentFieldKey.COVERARTMIME, "image/png"));
+            tag.setField(tag.createField(VorbisCommentFieldKey.COVERART, sb.toString()));
+            tag.setField(tag.createField(VorbisCommentFieldKey.COVERARTMIME, "image/png"));
 
             //Save
             f.commit();
@@ -610,8 +611,8 @@ public class VorbisWriteTagTest extends AbstractTestCase
             VorbisCommentTag tag = (VorbisCommentTag) f.getTag();
 
             //Delete Large Image
-            tag.deleteTagField(VorbisCommentFieldKey.COVERART);
-            tag.deleteTagField(VorbisCommentFieldKey.COVERARTMIME);
+            tag.deleteField(VorbisCommentFieldKey.COVERART);
+            tag.deleteField(VorbisCommentFieldKey.COVERARTMIME);
 
             //Save
             f.commit();
@@ -664,8 +665,8 @@ public class VorbisWriteTagTest extends AbstractTestCase
             VorbisCommentTag tag = (VorbisCommentTag) f.getTag();
 
             //Delete Large Image
-            tag.deleteTagField(VorbisCommentFieldKey.COVERART);
-            tag.deleteTagField(VorbisCommentFieldKey.COVERARTMIME);
+            tag.deleteField(VorbisCommentFieldKey.COVERART);
+            tag.deleteField(VorbisCommentFieldKey.COVERARTMIME);
 
             //Save
             f.commit();
@@ -717,9 +718,9 @@ public class VorbisWriteTagTest extends AbstractTestCase
             VorbisCommentTag tag = (VorbisCommentTag) f.getTag();
 
             //Delete Large Image
-            tag.deleteTagField(VorbisCommentFieldKey.ARTIST);
-            tag.deleteTagField(VorbisCommentFieldKey.COVERART);
-            tag.deleteTagField(VorbisCommentFieldKey.COVERARTMIME);
+            tag.deleteField(VorbisCommentFieldKey.ARTIST);
+            tag.deleteField(VorbisCommentFieldKey.COVERART);
+            tag.deleteField(VorbisCommentFieldKey.COVERARTMIME);
 
             //Save
             f.commit();
@@ -773,4 +774,49 @@ public class VorbisWriteTagTest extends AbstractTestCase
         assertTrue(f.getTag().isEmpty());
         assertEquals("jaudiotagger", ((VorbisCommentTag) f.getTag()).getVendor());
     }
+
+    public void testWriteMultipleFields() throws Exception
+    {
+        File testFile = AbstractTestCase.copyAudioToTmp("test.ogg", new File("testWriteMultiple.ogg"));
+        AudioFile f = AudioFileIO.read(testFile);
+        f.getTag().addField(FieldKey.ALBUM_ARTIST,"artist1");
+        f.getTag().addField(FieldKey.ALBUM_ARTIST,"artist2");
+        f.commit();
+        f = AudioFileIO.read(testFile);
+        List<TagField> tagFields = f.getTag().getFields(FieldKey.ALBUM_ARTIST);
+        assertEquals(tagFields.size(),2);
+    }
+
+     public void testDeleteFields() throws Exception
+    {
+        //Delete using generic key
+        File testFile = AbstractTestCase.copyAudioToTmp("test.ogg", new File("testDeleteFields.ogg"));
+        AudioFile f = AudioFileIO.read(testFile);
+        List<TagField> tagFields = f.getTag().getFields(FieldKey.ALBUM_ARTIST_SORT);
+        assertEquals(0,tagFields.size());
+        f.getTag().addField(FieldKey.ALBUM_ARTIST_SORT,"artist1");
+        f.getTag().addField(FieldKey.ALBUM_ARTIST_SORT,"artist2");
+        tagFields = f.getTag().getFields(FieldKey.ALBUM_ARTIST_SORT);
+        assertEquals(2,tagFields.size());
+        f.getTag().deleteField(FieldKey.ALBUM_ARTIST_SORT);
+        f.commit();
+
+        //Delete using flac id
+        f = AudioFileIO.read(testFile);
+        tagFields = f.getTag().getFields(FieldKey.ALBUM_ARTIST_SORT);
+        assertEquals(0,tagFields.size());
+        f.getTag().addField(FieldKey.ALBUM_ARTIST_SORT,"artist1");
+        f.getTag().addField(FieldKey.ALBUM_ARTIST_SORT,"artist2");
+        tagFields = f.getTag().getFields(FieldKey.ALBUM_ARTIST_SORT);
+        assertEquals(2,tagFields.size());
+        f.getTag().deleteField("ALBUMARTISTSORT");
+        tagFields = f.getTag().getFields(FieldKey.ALBUM_ARTIST_SORT);
+        assertEquals(0,tagFields.size());
+        f.commit();
+
+        f = AudioFileIO.read(testFile);
+        tagFields = f.getTag().getFields(FieldKey.ALBUM_ARTIST_SORT);
+        assertEquals(0,tagFields.size());
+
+    }
 }
