@@ -3,10 +3,10 @@ package org.jaudiotagger.tag.wma;
 import org.jaudiotagger.AbstractTestCase;
 import org.jaudiotagger.audio.AudioFile;
 import org.jaudiotagger.audio.AudioFileIO;
-import org.jaudiotagger.audio.asf.tag.AsfFieldKey;
-import org.jaudiotagger.audio.asf.tag.AsfTag;
-import org.jaudiotagger.audio.asf.tag.AsfTagCoverField;
-import org.jaudiotagger.audio.asf.tag.AsfTagTextField;
+import org.jaudiotagger.tag.asf.AsfFieldKey;
+import org.jaudiotagger.tag.asf.AsfTag;
+import org.jaudiotagger.tag.asf.AsfTagCoverField;
+import org.jaudiotagger.tag.asf.AsfTagTextField;
 import org.jaudiotagger.tag.FieldKey;
 import org.jaudiotagger.tag.Tag;
 import org.jaudiotagger.tag.TagField;
@@ -18,6 +18,7 @@ import java.awt.image.BufferedImage;
 import java.io.ByteArrayInputStream;
 import java.io.File;
 import java.io.RandomAccessFile;
+import java.util.List;
 
 /**
  * User: paul
@@ -281,7 +282,7 @@ public class WmaSimpleTest extends AbstractTestCase
             tag.setField(FieldKey.ALBUM,"album2");
             tag.setField(FieldKey.TITLE,"tracktitle2");
             tag.setField(FieldKey.COMMENT,"comments2");
-            tag.setField(FieldKey.YEAR,"1972");
+            tag.addField(FieldKey.YEAR,"1972");
             tag.setField(FieldKey.GENRE,"genre2");
             tag.setField(FieldKey.TRACK,"4");
             tag.setCopyright("copyright");
@@ -291,14 +292,14 @@ public class WmaSimpleTest extends AbstractTestCase
             tag.setField(tag.createField(FieldKey.URL_DISCOGS_RELEASE_SITE,"http://www.discogs2.com"));
             tag.setField(tag.createField(FieldKey.URL_OFFICIAL_ARTIST_SITE,"http://www.discogs3.com"));
             tag.setField(tag.createField(FieldKey.URL_OFFICIAL_RELEASE_SITE,"http://www.discogs4.com"));
-            tag.setField(tag.createField(FieldKey.URL_WIKIPEDIA_ARTIST_SITE,"http://www.discogs5.com"));
-            tag.setField(tag.createField(FieldKey.URL_WIKIPEDIA_RELEASE_SITE,"http://www.discogs6.com"));
+            tag.addField(tag.createField(FieldKey.URL_WIKIPEDIA_ARTIST_SITE,"http://www.discogs5.com"));
+            tag.addField(tag.createField(FieldKey.URL_WIKIPEDIA_RELEASE_SITE,"http://www.discogs6.com"));
             tag.setField(tag.createField(FieldKey.DISC_TOTAL,"3"));
             tag.setField(tag.createField(FieldKey.TRACK_TOTAL,"11"));
 
 
             // setField the IsVbr value (can be modified for now)
-            tag.setField(tag.createTagField(AsfFieldKey.ISVBR, Boolean.TRUE.toString()));
+            tag.setField(tag.createField(AsfFieldKey.ISVBR, Boolean.TRUE.toString()));
             f.commit();
 
             f = AudioFileIO.read(testFile);
@@ -854,7 +855,7 @@ public class WmaSimpleTest extends AbstractTestCase
             AsfTag asftag = (AsfTag) tag;
             asftag.setField(asftag.createArtworkField(imagedata));
             f.commit();
-
+                                             
             f = AudioFileIO.read(testFile);
             tag = f.getTag();
             assertEquals(1, tag.getFields(FieldKey.COVER_ART).size());
@@ -879,6 +880,52 @@ public class WmaSimpleTest extends AbstractTestCase
         assertNull(exceptionCaught);
     }
 
-    
+   /* TODO multiple fields for WMA
+
+    public void testWriteMultipleFields() throws Exception
+    {
+        File testFile = AbstractTestCase.copyAudioToTmp("test1.wma", new File("testWriteMultiple.wma"));
+        AudioFile f = AudioFileIO.read(testFile);
+        List<TagField> tagFields = f.getTag().getFields(FieldKey.ALBUM_ARTIST_SORT);
+        assertEquals(tagFields.size(),0);
+        f.getTag().addField(FieldKey.ALBUM_ARTIST_SORT,"artist1");
+        f.getTag().addField(FieldKey.ALBUM_ARTIST_SORT,"artist2");
+        f.commit();
+        f = AudioFileIO.read(testFile);
+        tagFields = f.getTag().getFields(FieldKey.ALBUM_ARTIST_SORT);
+        assertEquals(2,tagFields.size());
+    }
+    */
+
+    public void testDeleteFields() throws Exception
+    {
+        //Delete using generic key
+        File testFile = AbstractTestCase.copyAudioToTmp("test1.wma", new File("testDeleteFields.wma"));
+        AudioFile f = AudioFileIO.read(testFile);
+        List<TagField> tagFields = f.getTag().getFields(FieldKey.ALBUM_ARTIST_SORT);
+        assertEquals(0,tagFields.size());
+        f.getTag().addField(FieldKey.ALBUM_ARTIST_SORT,"artist1");
+        tagFields = f.getTag().getFields(FieldKey.ALBUM_ARTIST_SORT);
+        assertEquals(1,tagFields.size());
+        f.getTag().deleteField(FieldKey.ALBUM_ARTIST_SORT);
+        f.commit();
+
+        //Delete using flac id
+        f = AudioFileIO.read(testFile);
+        tagFields = f.getTag().getFields(FieldKey.ALBUM_ARTIST_SORT);
+        assertEquals(0,tagFields.size());
+        f.getTag().addField(FieldKey.ALBUM_ARTIST_SORT,"artist1");
+        tagFields = f.getTag().getFields(FieldKey.ALBUM_ARTIST_SORT);
+        assertEquals(1,tagFields.size());
+        f.getTag().deleteField("WM/AlbumArtistSortOrder");
+        tagFields = f.getTag().getFields(FieldKey.ALBUM_ARTIST_SORT);
+        assertEquals(0,tagFields.size());
+        f.commit();
+
+        f = AudioFileIO.read(testFile);
+        tagFields = f.getTag().getFields(FieldKey.ALBUM_ARTIST_SORT);
+        assertEquals(0,tagFields.size());
+
+    }
 }
 
